
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const CheckHardware_cxx_version =
    "$Id: CheckHardware.cxx,v 1.3 2002/04/05 08:07:41 manish Exp $";

#include "global.h"
#include "CheckHardware.hxx"
#include <linux/telephony.h>
#ifndef IXJUSER_H
#define IXJUSER_H
#include <linux/ixjuser.h>
#endif
#include <typeinfo>

///
CheckHardware::CheckHardware()
{
}

///
CheckHardware::~CheckHardware()
{
}

///

const char *
CheckHardware::GetType()
{
    const char *deviceName = RgConfiguration::instance()->getDeviceName().c_str();

    if (deviceName == "")
        return "NULL";

    // initialize ixj
    int ixj = 0;

    // open and close it to reset
    ixj = open(deviceName, O_RDWR);
    close(ixj);
    ixj = open(deviceName, O_RDWR);

    if (ixj <= 0)
    {
        cpLog(LOG_ERR, "Open failed, device: %s Error: %s", deviceName, strerror(errno));
        cpLog(LOG_ERR, "Defaulting to NULL_HARDWARE");
        return "NULL";
    }
    QnHardware qnDevice = QN_NULL;

    qnDevice = static_cast < QnHardware > (ioctl(ixj, IXJCTL_CARDTYPE));
    close(ixj);

    // what kind of quicknet card do we have?
    switch (qnDevice)
    {
        case QN_LINEJACK:
            {
                cpLog(LOG_DEBUG, "device is type LineJACK");
                return "QUICKNET";
            }
            break;

        default:
            {
                cpLog(LOG_ERR, "device is of type: %d. Not a LineJACK", qnDevice);
                cpLog(LOG_ERR, "defaulting to null hardware");
                return "NULL";
            }
            break;
    }                                   // end switch()

    return "NULL";
}
