#ifndef Contact_HXX
#define Contact_HXX

/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const ContactVersion =
    "$Id: Contact.hxx,v 1.2 2002/04/05 08:07:41 manish Exp $";

#include <InviteMsg.hxx>

#include <StatusMsg.hxx>

class InviteMsg;
class StatusMsg;
class SipUrl;
class SipCallLeg;
class SipStatusLine;

/// Keeps track of the one destination that we are trying
class Contact
{
  public:
    ///
    Contact(const Contact & c);

    ///
     Contact(const InviteMsg & msg);

    /// A update msg has arrived from this contact, may want to save some stuff
    void update(const StatusMsg & msg);

    /// the location being tried - is just the Request-URI part
     Sptr < BaseUrl > getLocation();

    ///get the call sequence number
    unsigned int getCSeqNum()
    {
        return cSeqNum;
    };

    ///
    void setCSeqNum(unsigned int cseq);

    ///
    InviteMsg getInviteMsg();

    ///
    StatusMsg getStatusMsg();

    /// The branch number of this attempt
    int getBranch();

    /// the best 3 digit SIP status code returned - initally 0
    int getStatus();

    /// Call leg for this attempt
    SipCallLeg computeCallLeg() const;

    ///
    void setLastInfoCSeq(unsigned int cseq);
    unsigned int getLastInfoCSeq();

    ///
    void setRemoteRingback(bool flag);
    bool getRemoteRingback() const;

    /// Check if the CallLeg, Location, and Branch match
    bool operator == (const SipMsg & msg);
    bool operator == (const Contact & other)const;

    /// oposite of == operator
    bool operator != (const Contact & other);

    ///
    void operator = (const Contact & src);

  private:
    unsigned int lastInfoCSeq;
    int status;
    unsigned int cSeqNum;

    InviteMsg inviteMsg;
    StatusMsg statusMsg;

    bool remoteRingback;
};

#endif
