
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const DigitTimeout_cxx_Version =
    "$Id: DigitTimeout.cxx,v 1.2 2002/04/05 08:07:41 manish Exp $";

#include "global.h"
#include <cassert>

#include "DigitTimeout.hxx"
#include "RgDevice.hxx"
#include "RgDigitTimerEvent.hxx"

///
DigitTimeout::DigitTimeout(Sptr < Fifo < Sptr < SipProxyEvent > > >cpq,
                           int initTimeout, int interDigitTimeout)
{
    initTimeoutVal = initTimeout;
    interDigitTimeoutVal = interDigitTimeout;
    isTimeout = false;
    callProcessingQueue = cpq;
    id = 0;

    //spawing the timeout thread
    digitTimeoutThread.spawn(digitTimeoutThreadWrapper, this);
    cpLog(LOG_DEBUG, "Spawning the interdigit timeout thread");
}

///
void
DigitTimeout::timerReset()
{
    digitTimeoutMutex.lock();
    isTimeout = false;
    digitTimeoutFifo.cancel(id);
    id = digitTimeoutFifo.addDelayMs(INTER_DIGIT_TIME_OUT, initTimeoutVal);
    digitTimeoutMutex.unlock();
}

///
void
DigitTimeout::timerStart()
{
    digitTimeoutMutex.lock();
    digitTimeoutFifo.cancel(id);
    id = digitTimeoutFifo.addDelayMs(INTER_DIGIT_TIME_OUT, interDigitTimeoutVal);
    digitTimeoutMutex.unlock();
}

///
void
DigitTimeout::timerCancel()
{
    digitTimeoutFifo.cancel(id);

    digitTimeoutMutex.lock();
    id = 0;
    isTimeout = false;
    digitTimeoutMutex.unlock();
}

///
void *
DigitTimeout::digitTimeoutThreadWrapper(void *timeout)
{
    assert(timeout);

    DigitTimeout *dTimeout = static_cast < DigitTimeout * >(timeout);

    dTimeout->digitTimeoutMain();
    return 0;
}

///
void
DigitTimeout::digitTimeoutMain()
{
    cpLog(LOG_DEBUG, "starting digitTimeout");
    while (true)
    {
        InterDigitTimeoutType digitTimeout = digitTimeoutFifo.getNext();

        if (digitTimeout == INTER_DIGIT_TIME_OUT)
        {
            digitTimeoutMutex.lock();
            isTimeout = true;
            digitTimeoutMutex.unlock();

            Sptr < RgDigitTimerEvent > timerEvent =
                new RgDigitTimerEvent(callProcessingQueue);
            assert(timerEvent != 0);
            timerEvent->callId = RgDevice::instance()->getCallId();
            callProcessingQueue->add(timerEvent);
        }
    }
}
