
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const QuickNetDevice_cxx_Version =
    "$Id: QuickNetDevice.cxx,v 1.6 2002/04/05 08:07:41 manish Exp $";

#include "global.h"

/* stdlib */
#include <iostream>
#include <cstdio>
#include <unistd.h>
#include <sys/ioctl.h>
#include <csignal>

/* sockets */
#include <sys/types.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <cerrno>

#include <linux/telephony.h>

#ifndef IXJUSER_H
#define IXJUSER_H
#include <linux/ixjuser.h>
#endif

#include "VTime.hxx"
#include "Rtp.hxx"
#include "VCondition.h"

/* error handling */

#include "QuickNetDevice.hxx"
#include "RgDeviceEvent.hxx"
#include "RgConfiguration.hxx"
//#include "RgOperator.hxx"

// #define ULAW_PAYLOAD 1012
#define ULAW_SAMPLE_RATE 240
#define RESID_RTP_RATE 240
#define NETWORK_RTP_RATE 160
#define FLASH_TIMEOUT 1000000

#define ANSWER 2                        // No of rings before call is answered.

int reportFlag = 0, nrings = 0, result, readbytes;      //readbytes for sdp

//fd_set readfds, exceptionfds;
fd_set exceptionfds;

//struct timeval tv;
unsigned long timeout;
char valDtmf;
char pstnnum[20];
static long int tv_previous = 0;
static long int tv_current = 0;
static int cadence_count = 0;
IXJ_FILTER filter0, filter1, filter2;

//IXJ_FILTER filter3;
IXJ_FILTER_CADENCE cadence0, cadence1, cadence2;

//IXJ_FILTER_CADENCE cadence3;
//IXJ_TONE *pTone;

//***************************************************************************
// QuickNetDevice::QuickNetDevice
// description:  Constructor.  opens the quicknet device.  initialize the
//               hookstate.  resets the device to get it ready.
//***************************************************************************

QuickNetDevice::QuickNetDevice(const char *deviceName,
                               Sptr < Fifo < Sptr < SipProxyEvent > > >inputQ,
                               Sptr < Fifo < Sptr < SipProxyEvent > > >outputQ):
ResGwDevice(deviceName, inputQ, outputQ),
audioStack(0),
inRtpPkt(0),
ringbackFd(-1),
sendRingback(false),
cwBeep(0),
cwCadenceOn(true),
cwCnt(0),
cwBeepOn(false)
{
    // initialize queues
    sessionQ = outputQ;
    myQ = inputQ;

    // open hardware device

    myFD = open(deviceName, O_RDWR);

    if (myFD < 0)
    {
        cpLog(LOG_ERR, "Cannot open %s", deviceName);
        exit(1);
    }

    // close and reopen to make sure device is not in a funny state
    // ioctl(myFD, IXJCTL_PORT, PORT_POTS);

    ioctl(myFD, PHONE_REC_STOP);
    ioctl(myFD, PHONE_PLAY_STOP);
    ioctl(myFD, IXJCTL_AEC_STOP);

    close(myFD);
    myFD = open(deviceName, O_RDWR);

    //ioctl(myFD, IXJCTL_PORT, PORT_POTS);

    if (myFD < 0)
    {
        cpLog(LOG_ERR, "Cannot re-open ", deviceName);
        exit(1);
    }

    // Set port to PSTN...
    if (ioctl(myFD, IXJCTL_PORT, PORT_PSTN))
    {
        cpLog(LOG_ERR, "Cannot set port to PSTN", deviceName);
        exit(1);
    }
    cpLog(LOG_DEBUG, "port set to PSTN");
    ioctl(myFD, IXJCTL_DSP_IDLE);

    // Test the Line...
    if (!(ioctl(myFD, IXJCTL_PSTN_LINETEST)))
    {
        cpLog(LOG_ERR, "Line not operational", deviceName);
        exit(1);
    }
    cpLog(LOG_DEBUG, "Line ready to receive calls");

    // Initialize the hookstate status to On Hook...
    if (ioctl(myFD, IXJCTL_PSTN_SET_STATE, PSTN_ON_HOOK))
    {
        cpLog(LOG_ERR, "Cannot initialize PSTN port to OnHook state", deviceName);
        exit(1);
    }
    hookStateOffhook = false;
    cpLog(LOG_DEBUG, "PSTN initialized to On Hook");

    // Set the tone On & Off times
    ioctl(myFD, PHONE_SET_TONE_ON_TIME, 0x02D0);        // 180 millisec
    ioctl(myFD, PHONE_SET_TONE_OFF_TIME, 0x00B4);       // 45 millisec

    /****************************************************************************

    pTone = malloc ( sizeof(IXJ_TONE));
    pTone->tone_index = 27;
    pTone->freq0 = hz16000;
    pTone->gain0 = 1;
    pTone->freq1 = 0;
    pTone->gain1 = 0;

    result = ioctl(myFD, IXJCTL_INIT_TONE, pTone);
    if(result == pTone->freq0)
        cpLog(LOG_DEBUG,"tone updated\n");
    else
        cpLog(LOG_DEBUG, "tone update failed");

    ****************************************************************************/

    //****************************************************************************
    //      This sets the digital scaling factor for the audio volume.
    //      This is used to control the audio volume on     the Internet PhoneJACK
    //      and the PSTN volume on the Internet LineJACK.
    //
    //      Scaling uses 0x0100 as the 1.0 value.0x0200 is 2 times normal volume,
    //      0x0080 is half the normal volume.
    //
    //      Note that the volume control is digital scaling. Errors will not
    //      generate clipping as in analog equipment,but rollover errors.
    //***************************************************************************/
    ioctl(myFD, IXJCTL_PLAY_VOLUME, 0x0200);
    ioctl(myFD, IXJCTL_REC_VOLUME, 0x0200);

    filter0.filter = 0;
    //filter0.freq = f400;        // Dial tone
    filter0.freq = RgConfiguration::instance()->getDialToneFreq();      // Dial tone
    filter0.enable = 1;

    cadence0.filter = 0;
    cadence0.enable = 1;
    cadence0.en_filter = 1;
    // cfg specifies the intervals in ms. Here, the unit is 10 ms. Hence dividing by 10.
    cadence0.on1 = (unsigned int)(RgConfiguration::instance()->getDialToneCadenceOn1() / float(10));
    cadence0.off1 = (unsigned int)(RgConfiguration::instance()->getDialToneCadenceOff1() / float(10));
    cadence0.on2 = (unsigned int)(RgConfiguration::instance()->getDialToneCadenceOn2() / float(10));
    cadence0.off2 = (unsigned int)(RgConfiguration::instance()->getDialToneCadenceOff2() / float(10));
    cadence0.on3 = (unsigned int)(RgConfiguration::instance()->getDialToneCadenceOn3() / float(10));
    cadence0.off3 = (unsigned int)(RgConfiguration::instance()->getDialToneCadenceOff3() / float(10));

    filter1.filter = 1;
    //filter1.freq = f400;        // Ringback
    filter1.freq = RgConfiguration::instance()->getRingbackToneFreq();  // Ringback tone
    filter1.enable = 1;

    cadence1.filter = 1;
    cadence1.enable = 1;
    cadence1.en_filter = 1;
    // cfg specifies the intervals in ms. Here, the unit is 10 ms. Hence dividing by 10.
    cadence1.on1 =
        (unsigned int)(RgConfiguration::instance()->getRingbackToneCadenceOn1() / float(10));
    cadence1.off1 =
        (unsigned int)(RgConfiguration::instance()->getRingbackToneCadenceOff1() / float(10));
    cadence1.on2 =
        (unsigned int)(RgConfiguration::instance()->getRingbackToneCadenceOn2() / float(10));
    cadence1.off2 =
        (unsigned int)(RgConfiguration::instance()->getRingbackToneCadenceOff2() / float(10));
    cadence1.on3 =
        (unsigned int)(RgConfiguration::instance()->getRingbackToneCadenceOn3() / float(10));
    cadence1.off3 =
        (unsigned int)(RgConfiguration::instance()->getRingbackToneCadenceOff3() / float(10));

    filter2.filter = 2;
    //filter2.freq = f400;        // Busy
    filter2.freq = RgConfiguration::instance()->getBusyToneFreq();      // Busy tone
    filter2.enable = 1;

    cadence2.filter = 2;
    cadence2.enable = 1;
    cadence2.en_filter = 1;
    // cfg specifies the intervals in ms. Here, the unit is 10 ms. Hence dividing by 10.
    cadence2.on1 = (unsigned int)(RgConfiguration::instance()->getBusyToneCadenceOn1() / float(10));
    cadence2.off1 = (unsigned int)(RgConfiguration::instance()->getBusyToneCadenceOff1() / float(10));
    cadence2.on2 = (unsigned int)(RgConfiguration::instance()->getBusyToneCadenceOn2() / float(10));
    cadence2.off2 = (unsigned int)(RgConfiguration::instance()->getBusyToneCadenceOff2() / float(10));
    cadence2.on3 = (unsigned int)(RgConfiguration::instance()->getBusyToneCadenceOn3() / float(10));
    cadence2.off3 = (unsigned int)(RgConfiguration::instance()->getBusyToneCadenceOff3() / float(10));

    // Check the current hookstate status...
    if (ioctl(myFD, IXJCTL_PSTN_GET_STATE) == PSTN_OFF_HOOK)
        hookStateOffhook = true;
    else
        hookStateOffhook = false;

    // store the device name.
    myDeviceName = deviceName;

    readbytes = RESID_RTP_RATE;         // readbytes is for sdp nego.

}                                       // end QuickNetDevice::QuickNetDevice()

//***************************************************************************
// QuickNetDevice::~QuickNetDevice
// description:  Destructor
//***************************************************************************

QuickNetDevice::~QuickNetDevice()
{
    close(myFD);
}                                       // end QuickNetDevice::~QuickNetDevice()

//***************************************************************************
// QuickNetDevice::hardwareMain
// description:  main processing loop of the hardware
//***************************************************************************

void *
QuickNetDevice::hardwareMain(void *parms)
{
    // process forever on behalf of QuickNetDevice hardware
    fd_set readfds;
    struct timeval tv;
    int retval;
    int maxFd = 128;            // should be 0 then increment as needed when addToFdSet

    // reset file descriptors
    FD_ZERO(&readfds);
    FD_ZERO(&exceptionfds);

    addToFdSet(&readfds);
    /**
     * block on select for asyncronous events, but poll to process
     * audio and signal requests from endpoints in message queue
     * The following values were optimized by trial and error for
     * the quicknet ixj-0.3.18gpl drivers.
     */
    tv.tv_sec = 0;
    tv.tv_usec = 300;

//    if((retval = select(maxFd, &readfds, 0, 0, &tv)) < 0)
    if ((retval = select(maxFd, &readfds, 0, &exceptionfds, &tv)) < 0)
    {
        cpLog(LOG_ERR, "select() returned error %d", retval);
    }
    else
    {
        // if(process(&readfds) < 0)
        if (processPSTN(&exceptionfds) < 0)
        {
            cpLog(LOG_ERR, "hardware encountered an error");
            assert(0);
        }
    }

    if (myQ->size() > 0)
    {
        cpLog(LOG_DEBUG, "Signal from call control");
        processSessionMsg(myQ->getNext());
    }

    return 0;
}                                       // end QuickNetDevice::hardwareMain()

//***************************************************************************
// QuickNetDevice::processRTP
// description:  main processing loop for RTP
//***************************************************************************

void
QuickNetDevice::processRTP()
{

    deviceMutex.lock();
    if (audioStack == 0)
    {
        vusleep(10000);
        deviceMutex.unlock();
        return;
    }

    RtpSessionState sessionState = audioStack->getSessionState();

    //only process receive if receive is enabled
    if (sessionState == rtp_session_recvonly || sessionState == rtp_session_sendrecv)
    {

        inRtpPkt = audioStack->receive();
        if (inRtpPkt)
        {
            if (inRtpPkt->getPayloadUsage() != RESID_RTP_RATE)
                cpLog(LOG_DEBUG, "RtpStack API size doesn't match RESID_RTP_RATE");
            char *vBuf;

            if (cwBeepOn)
            {
                vBuf = getCwBeep();
                if (vBuf == 0)
                {
                    vBuf = inRtpPkt->getPayloadLoc();
                    write(myFD, vBuf, inRtpPkt->getPayloadUsage());
                }
                else
                {
                    write(myFD, vBuf, RESID_RTP_RATE);
                }
            }
            else
            {
                vBuf = inRtpPkt->getPayloadLoc();
                write(myFD, vBuf, inRtpPkt->getPayloadUsage());
            }
            //write (myFD, vBuf, RESID_RTP_RATE);

            // need to delete since RTP stack doesn't do it any more
            delete inRtpPkt;

            inRtpPkt = NULL;
        }
    }
    if (sessionState == rtp_session_sendonly || sessionState == rtp_session_sendrecv)
    {
        // cc = number of samples
        int cc;

        if (sendRingback)
        {
            cc = getRingbackTone(outBufferPkt, RESID_RTP_RATE);
        }
        else
        {
            // cc = read(myFD, outBufferPkt, RESID_RTP_RATE);
            cc = read(myFD, outBufferPkt, readbytes);
        }

        if (audioStack)
        {
            audioStack->transmitRaw(outBufferPkt, cc);
        }
    }

    if (audioStack &&
        sessionState != rtp_session_inactive && sessionState != rtp_session_undefined)
    {
        audioStack->processRTCP();
    }
    deviceMutex.unlock();
}

int
QuickNetDevice::addToFdSet(fd_set * fd)
{
    if ((audioActive || audioHalfActive) && audioStack)
    {
        RtpSessionState sessionState = audioStack->getSessionState();

        if (sessionState == rtp_session_recvonly || sessionState == rtp_session_sendrecv)
        {
            FD_SET((audioStack->getRtpRecv())->getSocketFD(), fd);
            FD_SET((audioStack->getRtcpRecv())->getSocketFD(), fd);
        }
    }

    FD_SET(myFD, fd);                   //for reading voice data from card
    FD_SET(myFD, &exceptionfds);        //for signal trapping from card

    return 0;
}                                       // end QuickNetDevice::addToFdSet()

//***************************************************************************
// QuickNetDevice::getRtpPort()
//
// description:  creates a new rtp session and reserves a rtp port
//***************************************************************************

int
QuickNetDevice::getRtpPort()
{
    //this is an arbitrarily defined number
    const int HIGH_RTP_PORT = RgConfiguration::instance()->getMaxRtpPort();
    const int LOW_RTP_PORT = RgConfiguration::instance()->getMinRtpPort();

    int port = 0;
    int lowPort = LOW_RTP_PORT;

    deviceMutex.lock();

    // create a rtp session if there is no existing session alread
    // this Rtp session will be idle
    if (audioStack == 0)
    {
        audioStack = new RtpSession(0);
    }

    while (1)
    {
        // resever a rtp port
        port = audioStack->reserveRtpPort(lowPort, HIGH_RTP_PORT);
        lowPort = (port > lowPort) ? port : lowPort;

        // attempt to reserve a rtcp port on port number higher than
        // the rtp port
        if (port != 0 && audioStack->reserveRtcpPort(port + 1, 0) != 0)
        {
            cpLog(LOG_DEBUG, "Reserved rtp/rtcp ports %d/%d", port, port + 1);
            break;
        }

        // if not successful allocating rtcp port, increment the minimum rtp
        // port and try again.  If lowPort reaches HIGH_RTP_PORT, return 0
        // to indicate port allocation failed.
        lowPort += 2;
        if (lowPort > HIGH_RTP_PORT)
        {
            port = 0;
            break;
        }
    }

    deviceMutex.unlock();

    return port;
}

//***************************************************************************
// QuickNetDevice::releaseRtpPort()
//
// description: destroy the rtp session and release the rtp port
//***************************************************************************

void
QuickNetDevice::releaseRtpPort()
{
    deviceMutex.lock();

    // destroy the rtp session if one exists
    if (audioStack != 0)
    {
        int port = audioStack->releaseRtpPort();

        cpLog(LOG_DEBUG, "rtp port %d released", port);
        port = audioStack->releaseRtcpPort();
        cpLog(LOG_DEBUG, "rtcp port %d released", port);
        delete audioStack;

        audioStack = 0;
    }

    deviceMutex.unlock();
}

//***************************************************************************
// QuickNetDevice::audioStart
//
// description:  creates a new rtp session and also allocates memory for
//               incoming and outgoing rtp packet.  ioctl calls to initialize
//               the quicknet card.
//***************************************************************************

int
QuickNetDevice::audioStart(const HardwareAudioRequest & request)
{
    deviceMutex.lock();

    cpLog(LOG_DEBUG, "> > Establishing audio");
    cpLog(LOG_DEBUG, "> > Listening on port: %d", request.localPort);
    cpLog(LOG_DEBUG, "> > Sending to host  : %s", request.remoteHost);
    cpLog(LOG_DEBUG, "> > Sending to port  : %d", request.remotePort);
    cpLog(LOG_DEBUG, "> > RTP packet size  : %d ms", request.rtpPacketSize);

    RtpPayloadType apiFormat;
    phone_codec apicodec;

    switch (request.networkFormat)
    {
        case rtpPayloadPCMU:
            cpLog(LOG_INFO, "Setting PCMU on Quicknet");
            apiFormat = rtpPayloadPCMU;
            apicodec = ULAW;
            readbytes = 240;
            break;
        case rtpPayloadG723:
            cpLog(LOG_INFO, "Setting G723 on Quicknet");
            apiFormat = rtpPayloadG723;
            apicodec = G723_63;
            readbytes = 24;
            break;
        case rtpPayloadG726_32:
            cpLog(LOG_INFO, "g726 not available on hardware");
        default:
            cpLog(LOG_INFO, "Defaulting to PCMU on Quicknet");
            apiFormat = rtpPayloadPCMU;
            apicodec = ULAW;
            readbytes = 240;
    }

    if (audioStack == 0)
    {
        int remoteRtcpPort = (request.remotePort > 0) ? request.remotePort + 1 : 0;
        int localRtcpPort = (request.localPort > 0) ? request.localPort + 1 : 0;

        cpLog(LOG_DEBUG, "Remote RTCP port : %d", remoteRtcpPort);
        cpLog(LOG_DEBUG, "Local RTCP port  : %d", localRtcpPort);
        const char *remoteHost = 0;

        if (request.remotePort != 0)
            remoteHost = request.remoteHost;
        audioStack = new RtpSession(remoteHost, request.remotePort,
                                    request.localPort, remoteRtcpPort,
                                    localRtcpPort, rtpPayloadPCMU, rtpPayloadPCMU, 5);

        // ioctl(myFD, PHONE_REC_CODEC, ULAW);
        ioctl(myFD, PHONE_REC_CODEC, apicodec);
        ioctl(myFD, PHONE_REC_START);

        // ioctl(myFD, PHONE_PLAY_CODEC, ULAW);
        ioctl(myFD, PHONE_PLAY_CODEC, apicodec);
        ioctl(myFD, PHONE_PLAY_START);

        ioctl(myFD, IXJCTL_AEC_START, 1);
    }
    else
    {
        cpLog(LOG_DEBUG, "Modifying existing audio connection");

        //need to determine what mode RtpSession is in
        RtpSessionState sessionState = audioStack->getSessionState();

        switch (sessionState)
        {
            case rtp_session_sendonly:
                cpLog(LOG_DEBUG, "Session sendonly");
                //turn on the receive
                if (request.localPort != 0)
                {
                    cpLog(LOG_DEBUG, "turning on receive");
                    cpLog(LOG_DEBUG, "local host: %d", request.localPort);
                    cpLog(LOG_DEBUG, "local port: %d", request.localPort);
                    audioStack->setSessionState(rtp_session_sendrecv);
                    audioStack->setReceiver(request.localPort, request.localPort + 1, 0,
                                            apiFormat, request.networkFormat, 5);
                }
                //restart the quicknet card
                ioctl(myFD, PHONE_REC_STOP);
                ioctl(myFD, PHONE_PLAY_STOP);

                // ioctl(myFD, PHONE_REC_CODEC, ULAW);
                ioctl(myFD, PHONE_REC_CODEC, apicodec);
                ioctl(myFD, PHONE_REC_START);

                // ioctl(myFD, PHONE_PLAY_CODEC, ULAW);
                ioctl(myFD, PHONE_PLAY_CODEC, apicodec);
                ioctl(myFD, PHONE_PLAY_START);
                break;
            case rtp_session_recvonly:
                cpLog(LOG_DEBUG, "Session recvonly");
                //turn on the send
                cpLog(LOG_DEBUG, "turning on the send");
                cpLog(LOG_DEBUG, "remote host: %d", request.remoteHost);
                cpLog(LOG_DEBUG, "remote port: %d", request.remotePort);
                if (request.remotePort != 0)
                {
                    audioStack->setSessionState(rtp_session_sendrecv);
                    audioStack->setTransmiter(request.remoteHost, request.remotePort,
                                              request.remotePort + 1, apiFormat,
                                              request.networkFormat);
                }
                //restart the quicknet card
                ioctl(myFD, PHONE_REC_STOP);
                ioctl(myFD, PHONE_PLAY_STOP);

                // ioctl(myFD, PHONE_REC_CODEC, ULAW);
                ioctl(myFD, PHONE_REC_CODEC, apicodec);
                ioctl(myFD, PHONE_REC_START);

                // ioctl(myFD, PHONE_PLAY_CODEC, ULAW);
                ioctl(myFD, PHONE_PLAY_CODEC, apicodec);
                ioctl(myFD, PHONE_PLAY_START);
                break;
            case rtp_session_sendrecv:
                cpLog(LOG_DEBUG, "Session sendrecv");
                return 0;
                break;
            default:

                ioctl(myFD, PHONE_PLAY_STOP);
                ioctl(myFD, PHONE_REC_STOP);

                if (request.remotePort != 0 && request.localPort != 0)
                {
                    audioStack->setSessionState(rtp_session_sendrecv);
                    cpLog(LOG_DEBUG, "Session state is sendrecv");
                }
                else if (request.localPort != 0)
                {
                    audioStack->setSessionState(rtp_session_recvonly);
                    cpLog(LOG_DEBUG, "Session state is recvonly");
                }
                else if (request.remotePort != 0)
                {
                    audioStack->setSessionState(rtp_session_sendonly);
                    cpLog(LOG_DEBUG, "Session state is sendonly");
                }
                else
                {
                    cpLog(LOG_ERR, "Session state is undefined");
                }

                //turn on the receive
                if (request.localPort != 0)
                {
                    cpLog(LOG_DEBUG, "Turn on recv");

                    audioStack->setReceiver(request.localPort,
                                            request.localPort + 1,
                                            0, apiFormat, request.networkFormat, 5);

                    // ioctl(myFD, PHONE_PLAY_CODEC, ULAW);
                    ioctl(myFD, PHONE_PLAY_CODEC, apicodec);
                    ioctl(myFD, PHONE_PLAY_START);
                }
                //turn on the send
                if (request.remotePort != 0)
                {
                    cpLog(LOG_DEBUG, "Turn on send");

                    audioStack->setTransmiter(request.remoteHost,
                                              request.remotePort,
                                              request.remotePort + 1,
                                              apiFormat, request.networkFormat);

                    // ioctl(myFD, PHONE_REC_CODEC, ULAW);
                    ioctl(myFD, PHONE_REC_CODEC, apicodec);
                    ioctl(myFD, PHONE_REC_START);
                }

#if 0
                ioctl(myFD, PHONE_REC_STOP);
                ioctl(myFD, PHONE_PLAY_STOP);

                ioctl(myFD, PHONE_REC_CODEC, ULAW);
                ioctl(myFD, PHONE_REC_START);

                ioctl(myFD, PHONE_PLAY_CODEC, ULAW);
                ioctl(myFD, PHONE_PLAY_START);
#endif
                break;
        }
    }

    if (request.sendRingback)
        startSendRingback();
    else
        stopSendRingback();

    RtpSessionState sessionState = audioStack->getSessionState();

    if (sessionState == rtp_session_sendonly || sessionState == rtp_session_recvonly)
    {
        audioActive = false;
        audioHalfActive = true;
    }
    else
    {
        audioActive = true;
        audioHalfActive = false;
    }

    // inRtpPkt = 0;
    if (inRtpPkt)
    {
        delete inRtpPkt;

        inRtpPkt = 0;
    }

    audioStack->setApiFormat(apiFormat, RESID_RTP_RATE);
    audioStack->setNetworkFormat(request.networkFormat, request.rtpPacketSize * 8);

    deviceMutex.unlock();

    return 0;
}                                       // end QuickNetDevice::audioStart()

//***************************************************************************
// QuickNetDevice::audioStop
//
// description:  tears down audio.  cleans up by delete objects created when
//               audioStart was called...  audioStack, rtp packets.
//***************************************************************************

int
QuickNetDevice::audioStop()
{
    cpLog(LOG_DEBUG, "***   Stopping audio   ***");

    // mark audio as deactivated.
    audioActive = false;
    audioHalfActive = false;

    vusleep(200);

    // make hardware calls to stop audio
    deviceMutex.lock();
    ioctl(myFD, PHONE_REC_STOP);
    ioctl(myFD, PHONE_PLAY_STOP);
    ioctl(myFD, IXJCTL_AEC_STOP);

    // TBD, need to close and reopen device to stablize ipjack
    // hardware, hopefully this can be cleaned up with new driver
#if 0
    close(myFD);
    myFD = open(myDeviceName, O_RDWR);
    ioctl(myFD, IXJCTL_PORT, PORT_POTS);

    if (myFD < 0)
    {
        cpLog(LOG_ERR, "Cannot reopen %s", myDeviceName);
        exit(1);
    }
#endif

    // close RTP session
    if (audioStack)
    {
        RtpSessionState sessionState = audioStack->getSessionState();

        if (sessionState == rtp_session_sendonly || sessionState == rtp_session_sendrecv)
        {
            audioStack->transmitRTCPBYE();
        }
    }

    if (inRtpPkt)
    {
        delete inRtpPkt;

        inRtpPkt = 0;
    }

    if (audioStack)
    {
        delete audioStack;

        audioStack = 0;
    }

    deviceMutex.unlock();

    return 0;
}                                       // end QuickNetDevice::audioStop()

//***************************************************************************
// QuickNetDevice::audioSuspend
//
// description: suspend the RTP transmit and receive, and also stop the
//
//***************************************************************************

int
QuickNetDevice::audioSuspend()
{
    deviceMutex.lock();

    cpLog(LOG_INFO, "Suspending audio");
    cpLog(LOG_INFO, "Setting all RTP/RTCP ports to 0");

    if (audioStack != 0)
    {
        //set rtp session to inactive
        audioStack->setSessionState(rtp_session_inactive);
        audioStack->setReceiver(0, 0, 0, rtpPayloadPCMU, rtpPayloadPCMU, 5);
        audioStack->setTransmiter(0, 0, 0, rtpPayloadPCMU, rtpPayloadPCMU);
    }

    ioctl(myFD, PHONE_REC_STOP);
    ioctl(myFD, PHONE_PLAY_STOP);
    ioctl(myFD, IXJCTL_AEC_STOP);

    audioActive = false;
    audioHalfActive = false;

    deviceMutex.unlock();
    return 0;
}                                       // end QuickNetDevice::audioSuspend()

//***************************************************************************
// QuickNetDevice::audioResume
//
// description:  resume the rtp session using the new sdp information
//               enable the quicknet card audio
//***************************************************************************

int
QuickNetDevice::audioResume(const HardwareAudioRequest & request)
{
    deviceMutex.lock();

    if (audioStack == 0)
    {
        cpLog(LOG_ERR, "Try to resume an existing audio channel");
        cpLog(LOG_ERR, "No existing audio channel");
        deviceMutex.unlock();
        return 0;
    }

    cpLog(LOG_INFO, "Resuming audio");
    cpLog(LOG_INFO, "Listening on port: %d", request.localPort);
    cpLog(LOG_INFO, "Sending to host: %s", request.remoteHost);
    cpLog(LOG_INFO, "Sending to port: %d", request.remotePort);
    cpLog(LOG_INFO, "RTP packet size: %d", request.rtpPacketSize);

    //resuming audio channel
    //int remoteRtcpPort=(request.remotePort>0)?request.remotePort+1:0;
    //int localRtcpPort=(request.localPort>0)?request.localPort+1:0;
    const char *remoteHost = 0;

    if (request.remotePort != 0)
        remoteHost = request.remoteHost;

    if (request.localPort != 0 && request.remotePort != 0)
    {
        //full duplex audio
        audioStack->setSessionState(rtp_session_sendrecv);
        audioStack->setReceiver(request.localPort, request.localPort + 1, 0,
                                rtpPayloadPCMU, rtpPayloadPCMU, 5);
        audioStack->setTransmiter(request.remoteHost, request.remotePort,
                                  request.remotePort + 1, rtpPayloadPCMU, rtpPayloadPCMU);
        ioctl(myFD, PHONE_PLAY_CODEC, ULAW);
        ioctl(myFD, PHONE_PLAY_START);
        ioctl(myFD, PHONE_REC_CODEC, ULAW);
        ioctl(myFD, PHONE_REC_START);
        audioActive = true;
        audioHalfActive = false;
    }
    else if (request.localPort != 0)
    {
        //receive only
        audioStack->setSessionState(rtp_session_sendrecv);
        audioStack->setReceiver(request.localPort, request.localPort + 1, 0,
                                rtpPayloadPCMU, rtpPayloadPCMU, 5);
        ioctl(myFD, PHONE_PLAY_CODEC, ULAW);
        ioctl(myFD, PHONE_PLAY_START);
        audioActive = false;
        audioHalfActive = true;
    }
    else if (request.remotePort != 0)
    {
        //transmit only
        audioStack->setSessionState(rtp_session_sendrecv);
        audioStack->setTransmiter(request.remoteHost, request.remotePort,
                                  request.remotePort + 1, rtpPayloadPCMU, rtpPayloadPCMU);
        ioctl(myFD, PHONE_REC_CODEC, ULAW);
        ioctl(myFD, PHONE_REC_START);
        audioActive = false;
        audioHalfActive = true;
    }

#if 0
    // Echo Cancellation doesn't work with VM.
    if (request.echoCancellation)
    {
        ioctl(myFD, IXJCTL_AEC_START, 1);
        cerr << "echoCancellation is on" << endl;
    }
    else
    {
        cpLog(LOG_DEBUG, "echoCancellation is off");
        ioctl(myFD, IXJCTL_AEC_START, 0);
    }

    ioctl(myFD, PHONE_MIXER, 0x0C03);   // 06 is too soft
    ioctl(myFD, PHONE_MIXER, 0x0F01);   // 02 is too soft
#endif

    if (request.sendRingback)
        startSendRingback();
    else
        stopSendRingback();

    deviceMutex.unlock();
    return 0;
}                                       // end QuickNetDevice::audioResume()

// ****************
// ****************
int
QuickNetDevice::getRingbackTone(char *ringData, int size)
{
    struct timeval start;       // ctam
    struct timeval finish;      // ctam

    gettimeofday(&start, 0);            // ctam

    int cc = 0;

    char tone[RESID_RTP_RATE];

    if (ringbackFd == -1)
    {
        cpLog(LOG_INFO, "opening ringback tone to provide remote ringback");
        ringbackFd = open("Tone/ringback", O_RDONLY);
    }

    if (ringbackFd == -1)
    {
        cpLog(LOG_ERR, "can not open Tone/ringback");
        return cc;
    }

    cc = read(ringbackFd, tone, RESID_RTP_RATE);
    fflush(stdout);
#if 1
    while (cc < size)
    {
        lseek(ringbackFd, 0, SEEK_SET);
        cc += read(ringbackFd, tone + cc, size - cc);
    }
#endif
    memcpy(ringData, tone, size);

    long diff = 0;

    while (diff < 29500)
    {
        if (diff < 20000)
            vusleep(1000);
        gettimeofday(&finish, 0);       // ctam
        diff = finish.tv_usec - start.tv_usec + 1000000 * (finish.tv_sec - start.tv_sec);
    }

    return cc;
}

char *
QuickNetDevice::getCwBeep()
{
    char *beep = 0;

    if (cwCadenceOn)
    {
        beep = cwBeep + cwCnt;
        cwCnt += RESID_RTP_RATE;
        if (cwCnt >= CWBEEP_ON_TIME)
        {
            cwCnt = 0;
            cwCadenceOn = false;
        }
    }
    else
    {
        cwCnt += RESID_RTP_RATE;
        if (cwCnt >= (CWBEEP_OFF_TIME - RESID_RTP_RATE))
        {
            cwCnt = 0;
            cwCadenceOn = true;
        }
    }

    return beep;
}

void
QuickNetDevice::provideDialToneStart()
{
    cpLog(LOG_DEBUG, "***  Start dial tone  ***");
    deviceMutex.lock();
    ioctl(myFD, PHONE_DIALTONE);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideDialToneStop()
{
    cpLog(LOG_DEBUG, "***  Stop dial tone  ***");
    deviceMutex.lock();
    ioctl(myFD, PHONE_CPT_STOP);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideRingStart()
{
    cpLog(LOG_DEBUG, "***  Start Ringing  ***");
    deviceMutex.lock();
    ioctl(myFD, PHONE_RING_START);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideRingStop()
{
    cpLog(LOG_DEBUG, "***  Stop Ringing  ***");
    // no call to explicitly stop ringing is available at this time
    deviceMutex.lock();
    ioctl(myFD, PHONE_RING_STOP);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideLocalRingbackStart()
{
    cpLog(LOG_DEBUG, "***  Start local ringback  ***");
    deviceMutex.lock();
    ioctl(myFD, PHONE_RINGBACK);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideLocalRingbackStop()
{
    cpLog(LOG_DEBUG, "***  Stop local ringback  ***");
    deviceMutex.lock();
    ioctl(myFD, PHONE_CPT_STOP);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideBusyToneStart()
{
    cpLog(LOG_DEBUG, "***  Start busy tone  ***");
    deviceMutex.lock();
    ioctl(myFD, PHONE_BUSY);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideBusyToneStop()
{
    cpLog(LOG_DEBUG, "***  Stop busy tone  ***");
    deviceMutex.lock();
    ioctl(myFD, PHONE_CPT_STOP);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideFastBusyToneStart()
{
    cpLog(LOG_DEBUG, "***  Start fast busy tone  ***");
    deviceMutex.lock();
    //for now using regular busy tone
    ioctl(myFD, PHONE_BUSY);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideFastBusyToneStop()
{
    cpLog(LOG_DEBUG, "***  Stop fast busy tone  ***");
    deviceMutex.lock();
    //for now using regular busy tone
    ioctl(myFD, PHONE_CPT_STOP);
    deviceMutex.unlock();
}

void
QuickNetDevice::provideCallWaitingBeepStart()
{
    cpLog(LOG_DEBUG, "***  Start call waiting beep ***");

    //TODO: figure out how to do this with Quicknet
    //for now, just print something
    cout << "!!!!!!! Start call waiting beep" << endl;
    deviceMutex.lock();
    cwBeepOn = true;
    cwCnt = 0;
    cwCadenceOn = true;
    deviceMutex.unlock();
}

void
QuickNetDevice::provideCallWaitingBeepStop()
{
    cpLog(LOG_DEBUG, "***  Stop call waiting beep ***");

    //TODO: figure out how to do this with Quicknet
    //for now, just print something
    cout << "!!!!!!! Stop call waiting beep" << endl;
    cwBeepOn = false;
}

void
QuickNetDevice::provideDtmf(DeviceSignalType signal)
{
    char digit = ' ';
    int dtmfEvent = 0;

    switch (signal)
    {
        case DeviceSignalDigitStar:
            digit = '*';
            dtmfEvent = 10;
            break;
        case DeviceSignalDigitHash:
            digit = '#';
            dtmfEvent = 11;
            break;
        case DeviceSignalDigit0:
        case DeviceSignalDigit1:
        case DeviceSignalDigit2:
        case DeviceSignalDigit3:
        case DeviceSignalDigit4:
        case DeviceSignalDigit5:
        case DeviceSignalDigit6:
        case DeviceSignalDigit7:
        case DeviceSignalDigit8:
        case DeviceSignalDigit9:
            digit = '0' + signal - DeviceSignalDigit0;
            dtmfEvent = signal - DeviceSignalDigit0;
            break;
        default:
            cpLog(LOG_DEBUG, "Unknown DTMF Signal %d", signal);
            return;
    }

    if (audioStack)
    {
        cpLog(LOG_DEBUG, "DTMF %c sent out in RTP ", digit);
        audioStack->transmitEvent(dtmfEvent);
    }
    else
    {
        cpLog(LOG_DEBUG, "No audio stack existing to send DTMF event");
    }
}

void
QuickNetDevice::provideCallInfo(string, string, string)
{
}

// addded for IP-PSTN gateway.
//
//***************************************************************************
// QuickNetDevice::providePSTNOffHook
//
// description:  Sets the PSTN hook state to OFF_HOOK.
//***************************************************************************
void
QuickNetDevice::providePSTNOffHook()
{
    cpLog(LOG_DEBUG, "***  PSTN going Off Hook  ***");
    deviceMutex.lock();

    //go OffHook 
    if (ioctl(myFD, IXJCTL_PSTN_SET_STATE, PSTN_OFF_HOOK))
    {
        cpLog(LOG_ERR, "PSTN cannot go Off Hook");
        exit(1);
    }
    filter0.enable = 1;
    filter1.enable = 1;
    filter2.enable = 1;
    cadence0.enable = 1;
    cadence0.en_filter = 1;
    cadence1.enable = 1;
    cadence1.en_filter = 1;
    cadence2.enable = 1;
    cadence2.en_filter = 1;
    ioctl(myFD, IXJCTL_SET_FILTER, &filter0);
    ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence0);
    ioctl(myFD, IXJCTL_SET_FILTER, &filter1);
    ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence1);
    ioctl(myFD, IXJCTL_SET_FILTER, &filter2);
    ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence2);
    cadence_count = 0;
    deviceMutex.unlock();
};

//***************************************************************************
// QuickNetDevice::providePSTNOnHook
//
// description:  Sends the PSTN ON-HOOK.
//***************************************************************************
void
QuickNetDevice::providePSTNOnHook()
{
    cpLog(LOG_DEBUG, "***  PSTN going On Hook  ***");
    deviceMutex.lock();

    //go OnHook 
    if (ioctl(myFD, IXJCTL_PSTN_SET_STATE, PSTN_ON_HOOK))
    {
        cpLog(LOG_ERR, "PSTN cannot go On Hook");
        exit(1);
    }
    hookStateOffhook = false;
    cpLog(LOG_DEBUG, "PSTN initialized to On Hook");

    // Set the tone On & Off times
    ioctl(myFD, PHONE_SET_TONE_ON_TIME, 0x02D0);        // 180 millisec
    ioctl(myFD, PHONE_SET_TONE_OFF_TIME, 0x00B4);       // 45 millisec

    //resetting the filters..

    filter0.enable = 0;
    filter1.enable = 0;
    filter2.enable = 0;
    cadence0.enable = 0;
    cadence0.en_filter = 0;
    cadence1.enable = 0;
    cadence1.en_filter = 0;
    cadence2.enable = 0;
    cadence2.en_filter = 0;

    ioctl(myFD, IXJCTL_SET_FILTER, &filter0);
    ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence0);
    ioctl(myFD, IXJCTL_SET_FILTER, &filter1);
    ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence1);
    ioctl(myFD, IXJCTL_SET_FILTER, &filter2);
    ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence2);

    reportFlag = 0;
    tv_current = 0;
    tv_previous = 0;
    ioctl(myFD, IXJCTL_DSP_IDLE);
    vusleep(500);
    deviceMutex.unlock();
};

//***************************************************************************
// QuickNetDevice::providePSTNDtmf
//
// description: Converts the signal into the corresponding digits and 
//                              plays the digit on the PSTN.
//***************************************************************************
void
QuickNetDevice::providePSTNDtmf(DeviceSignalType signal)
{
    char cdigit[2];
    int digit;
    int state;

    cdigit[1]='\0';
    if (signal >= DeviceSignalPSTNDtmf0 && signal <= DeviceSignalPSTNDtmf9)
    {
        cdigit[0] = '0' + signal - DeviceSignalPSTNDtmf0;
    }
    else if (signal == DeviceSignalPSTNDtmfHash)
    {
        cdigit[0] = '#';
        digit = 12;
    }
    else if (signal == DeviceSignalPSTNDtmfStar)
    {
        cdigit[0] = '*';
        digit = 10;
    }

    digit = atoi(cdigit);
    cpLog(LOG_INFO, "Playing Tone %c", cdigit[0]);
    if (!digit)
    {
        digit = 11;
    }

    deviceMutex.lock();

    ioctl(myFD, IXJCTL_PLAY_TONE, digit);
    state = ioctl(myFD, IXJCTL_GET_TONE_STATE);

    while (state)
    {
        vusleep(40);
        state = ioctl(myFD, IXJCTL_GET_TONE_STATE);
    }
    deviceMutex.unlock();
};

//***************************************************************************
// QuickNetDevice::processPSTN
//
// description:  process any events detected on the hardware (dtmf digit
//               press, onhook & offhook, etc.) and reports the events back
//               to the session via the fifo queue.
//***************************************************************************

int
QuickNetDevice::processPSTN(fd_set * fd)
{
    union telephony_exception ixje1;
    unsigned short f0_hist, f1_hist, f2_hist, f3_hist;

    vusleep(0);                         // needed to stabilize hookstate readings

    deviceMutex.lock();

    if (FD_ISSET(myFD, fd))
    {
        ixje1.bytes = ioctl(myFD, IXJCTL_EXCEPTION);
        //cpLog( LOG_DEBUG, "Inside ProcessPSTN:inside if..exception read" );

        // for gettinf DTMF from pstn
        if (ixje1.bits.dtmf_ready)
        {
            ioctl(myFD, PHONE_CPT_STOP);
            valDtmf = ioctl(myFD, IXJCTL_GET_DTMF_ASCII);
            cpLog(LOG_DEBUG, "DTMF from card =%c", valDtmf);
            // new digit
            switch (valDtmf)
            {
                case '1':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf1);
                    }
                    break;

                case '2':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf2);
                    }
                    break;

                case '3':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf3);
                    }
                    break;

                case '4':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf4);
                    }
                    break;

                case '5':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf5);
                    }
                    break;

                case '6':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf6);
                    }
                    break;

                case '7':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf7);
                    }
                    break;

                case '8':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf8);
                    }
                    break;

                case '9':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf9);
                    }
                    break;

                case '*':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmfStar);
                    }
                    break;

                case '0':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmf0);
                    }
                    break;

                case '#':
                    {
                        reportEvent(sessionQ, DeviceEventPSTNDtmfHash);
                    }
                    break;

                default:
                    {
                        cpLog(LOG_ERR, "unrecognized DTMF digit");
                    }
                    break;
            }                           // end switch
        }

        //for pstn initiated call
        if (ixje1.bits.pstn_ring)
        {
            cpLog(LOG_DEBUG, "Ring %d detected on PSTN port", nrings);
            ++nrings;
            if (nrings >= ANSWER)
            {
                reportEvent(sessionQ, DeviceEventPSTNRinging);
            }
        }

        // pstn side disconnects
        if (ixje1.bits.pstn_wink)
        {
            cpLog(LOG_DEBUG, "Wink recieved from PSTN");
         /******************************************************************
		cout << "Wink Detected";
	    reportEvent(sessionQ, DeviceEventPSTNWink);
     *******************************************************************/
        }
        fflush(stdout);

        if (ixje1.bits.f0)
        {
            f0_hist = ioctl(myFD, IXJCTL_GET_FILTER_HIST, 0);
            if (!(f0_hist & 1))
            {
                // cpLog( LOG_DEBUG, "filter f0 (Dialtone) changed state OFF" );
            }
            else
            {
                //cpLog( LOG_DEBUG, "filter f0 (Dialtone) changed state ON" );
            }
        }

        if (ixje1.bits.fc0)
        {
            cpLog(LOG_DEBUG, "filter cadence 0 triggered (dialtone)=> tone detected");
            reportEvent(sessionQ, DeviceEventPSTNDialtone);
            fflush(stdout);
        }

        if (ixje1.bits.f1)
        {
            f1_hist = ioctl(myFD, IXJCTL_GET_FILTER_HIST, 1);
            if (!(f1_hist & 1))
            {
                //cpLog( LOG_DEBUG, "filter f1 (Ringback) changed state OFF" );
            }
            else
            {
                //cpLog( LOG_DEBUG, "filter f1 (Ringback) changed state ON" );
            }
        }

        if (ixje1.bits.fc1)
        {
            cpLog(LOG_DEBUG, "filter cadence 1 triggered (RingBack)=> tone detected");
                /*************************************************************************
		tv_previous = tv_current ;
		gettimeofday(&tv, NULL);
		tv_current = tv.tv_sec ;

		cpLog(LOG_DEBUG, "tv_current = %d",tv_current );
		// cout << "tv_current =%ld" << tv_current << endl ; 
		cpLog(LOG_DEBUG, "previous_tv_sec = %d",tv_previous );
		// cout << "tv_previous =%ld" << tv_previous << endl; 
		// // cout << "diff =%d" << (tv_current - tv_previous) << endl; 
		*************************************************************************/

            cadence_count++;
            cpLog(LOG_DEBUG, "RingBack count = %d", cadence_count);

            // For the first detection of cadence send PSTN ringback signal.
            if (cadence_count == 1)
            {
                reportEvent(sessionQ, DeviceEventPSTNRingback);
                cpLog(LOG_DEBUG, "RingBack event is reported");
                //ioctl(myFD, IXJCTL_SET_FILTER, &filter3);
                //ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence3);
            }

            // For the next detection need to start and stop the cadence timer.
            if (cadence_count < 9 && cadence_count > 1)
            {
                reportEvent(sessionQ, DeviceEventPSTNRingbackCadenceTimer);
                cpLog(LOG_DEBUG, "RingBack cadence Timer event is reported");
            }

            //If the cadence detection is more that 8 times..send no answer to the user.
            if (cadence_count > 8)
            {
                reportEvent(sessionQ, DeviceEventPSTNNoAnswer);
                cpLog(LOG_DEBUG, "PSTN NO answer event is reported");
            }

            ioctl(myFD, IXJCTL_SET_FILTER, &filter1);
            ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence1);
            ioctl(myFD, IXJCTL_SET_FILTER, &filter2);
            ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence2);
            fflush(stdout);
        }

        if (ixje1.bits.f2)
        {
            f2_hist = ioctl(myFD, IXJCTL_GET_FILTER_HIST, 2);
            if (!(f2_hist & 1))
            {
                //cpLog( LOG_DEBUG, "filter f2 (Busy) changed state OFF" );
            }
            else
            {
                //cpLog( LOG_DEBUG, "filter f2 (Busy) changed state ON" );
            }
        }

        if (ixje1.bits.fc2)
        {
            cpLog(LOG_DEBUG, "filter cadence 2 triggered (Busy)=> tone detected");
            reportEvent(sessionQ, DeviceEventPSTNBusy);
            ioctl(myFD, IXJCTL_SET_FILTER, &filter1);
            ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence1);
            ioctl(myFD, IXJCTL_SET_FILTER, &filter2);
            ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence2);
            fflush(stdout);
        }

    }

        /**********************************************************************************
	if(ixje1.bits.f3)
	{
	    f3_hist = ioctl(myFD, IXJCTL_GET_FILTER_HIST, 3);
	    if(!(f3_hist & 1))
	    {
	        //cpLog( LOG_DEBUG, "filter f3 (Wink) changed state OFF" );
	    }
	    else
	    {
	        //cpLog( LOG_DEBUG, "filter f3 (Wink) changed state ON" );
	    }
	}

   	if(ixje1.bits.fc3)
   	{
			reportEvent(sessionQ, DeviceEventPSTNWink);
		    ioctl(myFD, IXJCTL_SET_FILTER, &filter3);
		    ioctl(myFD, IXJCTL_FILTER_CADENCE, &cadence3);
	        fflush(stdout);  	
	}
	
    ****************************************************************************************/

    deviceMutex.unlock();
    return 0;
}
