
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const ResGateway_cxx_Version =
    "$Id: ResGateway.cxx,v 1.2 2002/04/05 08:07:41 manish Exp $";

#include "global.h"
#include "ResGateway.hxx"
#include "RgCommandLine.hxx"
#include "RgConfiguration.hxx"
#include "RgDevice.hxx"
#include "DeviceThread.hxx"
#include "RtpThread.hxx"
#include "RgCallContainer.hxx"

#include "CheckHardware.hxx"

#include "ResGwDevice.hxx"
#include "cpLog.h"

#include "RgDigitTimerEvent.hxx"

//extern int numOfCallsStarted;

///
ResGateway::ResGateway(Sptr < RgBuilder > rgBuilder, unsigned short sipPort,
                       Data appName):
HeartLessProxy(rgBuilder, sipPort, appName)
{
    const char *useDevice = "NULL";     // default to NULL_HARDWARE

    if (RgCommandLine::instance()->getBoolOpt("quicknet") &&
        RgCommandLine::instance()->getBoolOpt("voicemail") &&
        RgCommandLine::instance()->getBoolOpt("soundcard"))
    {
        cpLog(LOG_ERR, "Quicknet and Soundcard and Voicemail mutually exclusive");
        exit(-1);
    }

    if (RgCommandLine::instance()->getBoolOpt("quicknet"))
    {
        // then create a CheckHardware object
        CheckHardware check;

        useDevice = check.GetType();
    }
    // Create devices only if load gen is turned OFF.

    RgDevice::instance(useDevice, myCallProcessingQueue);

    vusleep(10000);
    if (!(strcmp(useDevice, "NONE") == 0))
    {
        cpLog(LOG_DEBUG, "Create RTP Thread");
        rtpThread = new RtpThread(RgDevice::instance());
        assert(rtpThread != 0);
    }

    cpLog(LOG_DEBUG, "Create Device Thread");
    deviceThread = new DeviceThread(RgDevice::instance());
    assert(deviceThread != 0);

    // Turn off retransmission
    if (RgCommandLine::instance()->getBoolOpt("retransmit"))
    {
        SipTransceiver::reTransOn();
    }
    else
    {
        SipTransceiver::reTransOff();
    }

    // Replace base CallContainer
    myCallContainer = new RgCallContainer;
    assert(myCallContainer != 0);
    rgBuilder->setCallContainer(myCallContainer);

    rgBuilder->setSipStack(mySipStack);

    cout << "READY" << endl;
}                                       // ResGateway::ResGateway

///
void
ResGateway::run()
{
    HeartLessProxy::run();

    if (deviceThread != 0)
    {
        cpLog(LOG_DEBUG, "Spawning Device Thread");
        deviceThread->run();
    }

    if (rtpThread != 0)
    {
        cpLog(LOG_DEBUG, "Spawning RTP Thread");
        rtpThread->run();
    }
}                                       // ResGateway::run

///
void
ResGateway::join()
{
    HeartLessProxy::join();
}                                       // ResGateway::join

///
ResGateway::~ResGateway()
{
}                                       // ResGateway::~ResGateway
