
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgBuilder_cxx_Version =
    "$Id: RgBuilder.cxx,v 1.3 2002/04/05 08:07:41 manish Exp $";

#include "global.h"
#include <SipEvent.hxx>
#include <StatusMsg.hxx>

#include "RgBuilder.hxx"
#include "RgCallContainer.hxx"
#include "RgStateMachine.hxx"
#include "RgDeviceEvent.hxx"
#include "RgDevice.hxx"
#include "RgDigitTimerEvent.hxx"
#include "RgTimerEvent.hxx"
#include "RgConfiguration.hxx"

///
RgBuilder::RgBuilder()
{
    Sptr < RgStateMachine > rgStateMachine = new RgStateMachine;
    addFeature(rgStateMachine);
}                                       // RgBuilder::RgBuilder

///
RgBuilder::~RgBuilder()
{
}                                       // RgBuilder::~RgBuilder

///
void
RgBuilder::process(const Sptr < SipProxyEvent > nextEvent)
{
    assert(nextEvent != 0);
    assert(myCallContainer != 0);

    calls.dynamicCast(myCallContainer);
    assert(calls != 0);

    stateMachine.dynamicCast(*(myFeatures.begin()));
    assert(stateMachine != 0);

    /// Handle the 4 types of events:

    /// SipEvent
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(nextEvent);
    if (sipEvent != 0)
    {
        /** Returning 'true' from a 'process' or 'handle' method causes
	 *  RgBuilder to exit without running sendEvent()
	 */
        if (processSipEvent(sipEvent))
        {
            cpLog(LOG_DEBUG, "Returning from processSipEvent\n");
            return;
        }
        sendEvent(nextEvent);
        return;
    }

    /// RgDeviceEvent
    Sptr < RgDeviceEvent > rgDeviceEvent;
    rgDeviceEvent.dynamicCast(nextEvent);
    if (rgDeviceEvent != 0)
    {
        if (processRgDeviceEvent(rgDeviceEvent))
        {
            cpLog(LOG_DEBUG, "Returning from processRgDeviceEvent\n");
            return;
        }
        sendEvent(nextEvent);
        return;
    }

    /// RgDigitEvent
    Sptr < RgDigitTimerEvent > rgDigitEvent;
    rgDigitEvent.dynamicCast(nextEvent);
    if (rgDigitEvent != 0)
    {
        if (processRgDigitEvent(rgDigitEvent))
        {
            cpLog(LOG_DEBUG, "Returning from processRgDigitEvent\n");
            return;
        }
        sendEvent(nextEvent);
        return;
    }

    /// RgTimerEvent
    Sptr < RgTimerEvent > rgTimerEvent;
    rgTimerEvent.dynamicCast(nextEvent);
    if (rgTimerEvent != 0)
    {
        if (processRgTimerEvent(rgTimerEvent))
        {
            cpLog(LOG_DEBUG, "Returning from processSipEvent\n");
            return;
        }
        sendEvent(nextEvent);
        return;
    }

    /// Event of unknown type
    // we shouldn't get to here.
    cpLog(LOG_ERR, "Unknown Event");
    assert(0);

}

/**
 * A parent 'process' method
 */
bool RgBuilder::processSipEvent(const Sptr < SipEvent > sipEvent)
{
    cpLog(LOG_DEBUG, "SIP message event");
    cpLog(LOG_DEBUG, "In processSipEvent()");

    Sptr < StatusMsg > statusMsg;
    statusMsg.dynamicCast(sipEvent->getSipMsg());
    /// If we have a status msg, we handle it
    if (statusMsg != 0)
    {
        if (handleStatusMsg(sipEvent))
        {
            return true;
        }
    }
    /// If we get here, we have a SIP message event
    cpLog(LOG_DEBUG, "SIP message event");
    /// Let's check the call info, now
    callId = sipEvent->getSipCallLeg()->getCallId();
    callInfo = calls->findCall(callId);

    if (callInfo == 0)
    {
        cpLog(LOG_DEBUG, "No CallInfo found");

        Sptr < InviteMsg > inviteMsg;
        inviteMsg.dynamicCast(sipEvent->getSipMsg());
        if (inviteMsg == 0)
        {
            cpLog(LOG_DEBUG, "Not an INVITE - Discard event");
            return true;
        }
        else
        {
            cpLog(LOG_DEBUG, "New INVITE");
            callInfo = calls->newCall(sipEvent->getSipCallLeg()->getCallId());
            assert(callInfo != 0);
            callInfo->setFeature(stateMachine);

            /// We're not in loadgen mode
            /// Handle call waiting conditions
            if (handleCallWaiting(callInfo))
            {
                cpLog(LOG_ERR, "Returned from handleCallWaiting\n");
                return true;
            }
        }                               // lots of brackets!
    }
    return false;
}                                       /// RgBuilder::processSipEvent

/**
 * A parent 'process' method
 */
bool RgBuilder::processRgDeviceEvent(const Sptr < RgDeviceEvent > rgDeviceEvent)
{
    cpLog(LOG_DEBUG, "Device Event");
    /// The following situtation can occur:
    ///   1.  user goes onhook
    ///   2.  user goes offhook immediately
    ///   3.  state machine processes the onhook and removes callid
    ///   4.  state machine processes the offhook with original callid
    ///   5.  the original callid no longer exists
    /// In this case, we will just change the callId in deviceEvent
    /// to the one in the device
    /// Short term fix only.

    bool
        exit =
        false;                  /// Tells us whether or not to exit RgBuilder

    if (!(rgDeviceEvent->callId == RgDevice::instance()->getCallId()))
    {
        rgDeviceEvent->callId = RgDevice::instance()->getCallId();
    }
    if (rgDeviceEvent->callId == 0)
    {
        if ((rgDeviceEvent->type == DeviceEventPSTNRinging))    // Adapted for gateway.
        {
            //cpLog( LOG_DEBUG, "User Offhook" );
            cpLog(LOG_DEBUG, "PSTN call ");
            Sptr < SipCallId > newCallId = new SipCallId;

            if (RgConfiguration::instance()->getCallIdOn())
            {
                // Use callId from config file
                Data
                newId(RgConfiguration::instance()->getCallId());

                newCallId->setLocalId(newId);
            }
            callInfo = calls->newCall(*newCallId);
            assert(callInfo != 0);
            callInfo->setFeature(stateMachine);
            callInfo->setState(stateMachine->findState("gwStateIdle"));
            RgDevice::instance()->setCallId(newCallId);
            rgDeviceEvent->setSipStack(sipStack);
        }
        else
            // We have no callId & the event is NOT an offhook
        {
            cpLog(LOG_ERR, "Missing Call Id for hardware event");
            exit = true;
            return exit;
        }
    }
    // We must have a valid callId to enter this else statement
    else
    {
        callInfo = calls->findCall(*(rgDeviceEvent->callId));
        rgDeviceEvent->setSipStack(sipStack);
        assert(callInfo != 0);
    }
    return exit;
}

/**
 * A parent 'process' method
 */
bool RgBuilder::processRgDigitEvent(const Sptr < RgDigitTimerEvent > rgDigitEvent)
{
    cpLog(LOG_DEBUG, "rg Digit Event");
    if (rgDigitEvent->callId == 0)
    {
        cpLog(LOG_DEBUG, "Discarded");
        return true;
    }
    else                                // rgDigitEvent->callId != 0 
    {
        callInfo = calls->findCall(*(rgDigitEvent->callId));
        if (callInfo == 0)
        {
            cpLog(LOG_DEBUG, "Discard late timer event");
            return true;
        }
        rgDigitEvent->setSipStack(sipStack);
    }
    return false;
}

/**
 * A parent 'process' method
 */
bool RgBuilder::processRgTimerEvent(const Sptr < RgTimerEvent > rgTimerEvent)
{
    Sptr < SipCallId > callId = rgTimerEvent->getCallId();
    assert(callId != 0);
    callInfo = calls->findCall(*callId);
    assert(callInfo != 0);
    rgTimerEvent->setSipStack(sipStack);
    return false;
}

/**
 * A child 'handle' method
 */
bool RgBuilder::handleStatusMsg(const Sptr < SipEvent > sipEvent)
{
    cpLog(LOG_DEBUG, "In handleStatusMsg()");
    /// Pull the StatusMsg out of the SipEvent
    Sptr < StatusMsg > statusMsg;
    statusMsg.dynamicCast(sipEvent->getSipMsg());
    /// Sanity check; this checked out ok in main process method
    assert(statusMsg != 0);
    return false;                       // a return to the calling method
}                                       /// end handleStatusMsg()

/**
 * A child 'handle' method
 */
bool RgBuilder::handleCallWaiting(Sptr < CallInfo > callInfo)
{
    /// call waiting is disabled. The new call will not ring
    /// the set when there is an active call
    if (RgDevice::instance()->getHardwareStatus() == HARDWARE_AVAILABLE)
    {
        cpLog(LOG_DEBUG, "Hardware Available");
        // New Call
        callInfo->setState(stateMachine->findState("gwStateIdle"));
    }
    else
    {
        cpLog(LOG_DEBUG, "Hardware Unavailable");
        // Device is in use
        callInfo->setState(stateMachine->findState("gwStateBusyHere"));
    }
    return false;
}

/**
 */
void
RgBuilder::sendEvent(const Sptr < SipProxyEvent > nextEvent)
{
    nextEvent->setCallInfo(callInfo, calls);
    stateMachine->process(nextEvent);

    return;
}

///
void
RgBuilder::setSipStack(const Sptr < SipTransceiver > sipstack)
{
    sipStack = sipstack;
}
