
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgCallInfo_cxx_Version =
    "$Id: RgCallInfo.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include "global.h"
#include "RgCallInfo.hxx"

///
RgCallInfo::RgCallInfo()
{
    originator = false;
    contactList = new ContactList;
    route1List = new RouteList;
    route2List = new RouteList;
    xferMsg = 0;
    ackMsg = 0;
    ringInvite = 0;
    ring2Invite = 0;
    currentContact = 0;
    localSdp = 0;
    remoteSdp = 0;
    local2Sdp = 0;
    remote2Sdp = 0;
}

///
RgCallInfo::~RgCallInfo()
{
}

///

void
RgCallInfo::setOriginator(const bool yorn)
{
    originator = yorn;
}

///
bool RgCallInfo::isOriginator()
{
    return originator;
}

///
void
RgCallInfo::setCallee(const SipUrl url)
{
    callee = url;
}

///
SipUrl RgCallInfo::getCallee()
{
    return callee;
}

///
Sptr < Contact > RgCallInfo::getContact()
{
    return currentContact;
}

///
void
RgCallInfo::setContact(Sptr < Contact > contact)
{
    currentContact = contact;
}

///
Sptr < ContactList > RgCallInfo::getContactList()
{
    return contactList;
}

///
void
RgCallInfo::addContact(const Sptr < Contact > contact)
{
    contactList->push_back(contact);
}

///
Sptr < Contact > RgCallInfo::findContact(const SipMsg & msg)
{
#if 0
    ContactList::iterator iter = contactList->begin();
    while (iter != contactList->end())
    {
        if ((*(*iter)).computeCallLeg() == msg.computeCallLeg())
        {
            return (*iter);
        }
        iter++;
    }
#else
    ContactList::iterator iter = contactList->end();
    while (iter != contactList->begin())
    {
        iter--;
        if ((*(*iter)).computeCallLeg() == msg.computeCallLeg())
        {
            return (*iter);
        }
    }
#endif
    return 0;
}

///
void
RgCallInfo::removeContact(const Sptr < Contact > contact)
{
    ContactList::iterator iter = contactList->begin();
    while (iter != contactList->end())
    {
        if (*(*iter) == *contact)
        {
            contactList->erase(iter);
            break;
        }
        iter++;
    }

    return;
}

///
void
RgCallInfo::clearContactList()
{
    contactList->clear();
}

///
void
RgCallInfo::setCaller(const SipUrl url)
{
    caller = url;
}

///
SipUrl RgCallInfo::getCaller()
{
    return caller;
}

///
void
RgCallInfo::setLocalSdp(const Sptr < SipSdp > sdp)
{
    localSdp = sdp;
}

///
void
RgCallInfo::setRemoteSdp(const Sptr < SipSdp > sdp)
{
    remoteSdp = sdp;
}

///
Sptr < SipSdp > RgCallInfo::getLocalSdp()
{
    return localSdp;
}

///
Sptr < SipSdp > RgCallInfo::getRemoteSdp()
{
    return remoteSdp;
}

///
void
RgCallInfo::setLocal2Sdp(const Sptr < SipSdp > sdp)
{
    local2Sdp = sdp;
}

///
void
RgCallInfo::setRemote2Sdp(const Sptr < SipSdp > sdp)
{
    remote2Sdp = sdp;
}

///
Sptr < SipSdp > RgCallInfo::getLocal2Sdp()
{
    return local2Sdp;
}

///
Sptr < SipSdp > RgCallInfo::getRemote2Sdp()
{
    return remote2Sdp;
}

///
void
RgCallInfo::setRingInvite(Sptr < InviteMsg > invite)
{
    ringInvite = invite;
}

///
Sptr < InviteMsg > RgCallInfo::getRingInvite()
{
    return ringInvite;
}

///
void
RgCallInfo::setRing2Invite(Sptr < InviteMsg > invite)
{
    ring2Invite = invite;
}

///
Sptr < InviteMsg > RgCallInfo::getRing2Invite()
{
    return ring2Invite;
}

///
void
RgCallInfo::setCallerRoute1List(SipRecordRouteList recordrouteList)
{
    route1List->clear();
    SipRecordRouteList::iterator iter = recordrouteList.begin();
    while (iter != recordrouteList.end())
    {
        Sptr < BaseUrl > baseUrl = (*iter)->getUrl();
        if (baseUrl->getType() == TEL_URL)
        {
            cpLog(LOG_ERR, "TEL_URL currently not supported\n");
            assert(0);
        }
        // Assume we have a SIP_URL
        Sptr < SipUrl > sipUrl;
        sipUrl.dynamicCast(baseUrl);
        assert(sipUrl != 0);

        cpLog(LOG_DEBUG, "setCallerRoute1List %s:%s",
              sipUrl->getHost().getData(), sipUrl->getPort().getData());
        Sptr < SipRoute > route = new SipRoute;
        route->setUrl(baseUrl);
        route1List->insert(route1List->begin(), route);
        iter++;
    }
}

///
void
RgCallInfo::setCalleeRoute1List(SipRecordRouteList recordrouteList)
{
    route1List->clear();
    SipRecordRouteList::iterator iter = recordrouteList.begin();
    while (iter != recordrouteList.end())
    {
        Sptr < BaseUrl > baseUrl = (*iter)->getUrl();
        if (baseUrl->getType() == TEL_URL)
        {
            cpLog(LOG_ERR, "TEL_URL currently not supported\n");
            assert(0);
        }
        // Assume we have a SIP_URL
        Sptr < SipUrl > sipUrl;
        sipUrl.dynamicCast(baseUrl);
        assert(sipUrl != 0);

        cpLog(LOG_DEBUG, "setCalleeRoute1List %s:%s",
              sipUrl->getHost().getData(), sipUrl->getPort().getData());

        Sptr < SipRoute > route = new SipRoute;
        route->setUrl(baseUrl);
        route1List->push_back(route);
        iter++;
    }
}

void
RgCallInfo::addRoute1(const Sptr < SipRoute > sipRoute)
{
    route1List->push_back(sipRoute);
}

///
vector < SipRoute * >RgCallInfo::getRoute1List()
{
    vector < SipRoute * >siprouteList;

    RouteList::iterator iter = route1List->begin();

    while (iter != route1List->end())
    {
        Sptr < BaseUrl > baseUrl = (*iter)->getUrl();
        if (baseUrl->getType() == TEL_URL)
        {
            cpLog(LOG_ERR, "TEL_URL currently not supported\n");
            assert(0);
        }
        // Assume we have a SIP_URL
        Sptr < SipUrl > sipUrl;
        sipUrl.dynamicCast(baseUrl);
        assert(sipUrl != 0);

        cpLog(LOG_DEBUG, "getRoute1List %s:%s",
              sipUrl->getHost().getData(), sipUrl->getPort().getData());

        siprouteList.push_back(new SipRoute(*(*iter)));
        iter++;
    }
    return siprouteList;
}

void
RgCallInfo::setCallerRoute2List(SipRecordRouteList recordrouteList)
{
    route2List->clear();
    SipRecordRouteList::iterator iter = recordrouteList.begin();
    while (iter != recordrouteList.end())
    {
        Sptr < SipRoute > route = new SipRoute;
        route->setUrl((*iter)->getUrl());
        route2List->insert(route2List->begin(), route);
        iter++;
    }
}

void
RgCallInfo::setCalleeRoute2List(SipRecordRouteList recordrouteList)
{
    route2List->clear();
    SipRecordRouteList::iterator iter = recordrouteList.begin();
    while (iter != recordrouteList.end())
    {
        Sptr < SipRoute > route = new SipRoute;
        route->setUrl((*iter)->getUrl());
        route2List->push_back(route);
        iter++;
    }
}

void
RgCallInfo::addRoute2(const Sptr < SipRoute > sipRoute)
{
    route2List->push_back(sipRoute);
}

///
vector < SipRoute * >RgCallInfo::getRoute2List()
{
    vector < SipRoute * >siprouteList;

    RouteList::iterator iter = route2List->begin();

    while (iter != route2List->end())
    {
        siprouteList.push_back(new SipRoute(*(*iter)));
        iter++;
    }
    return siprouteList;
}

///
void
RgCallInfo::setXferMsg(Sptr < TransferMsg > msg)
{
    xferMsg = msg;
}

///
Sptr < TransferMsg > RgCallInfo::getXferMsg()
{
    return xferMsg;
}

///
void
RgCallInfo::setAckMsg(Sptr < AckMsg > ack)
{
    ackMsg = ack;
}

///
Sptr < AckMsg > RgCallInfo::getAckMsg()
{
    return ackMsg;
}
