#ifndef RgCallInfo_H
#define RgCallInfo_H

/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgCallInfoVersion =
    "$Id: RgCallInfo.hxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <Sptr.hxx>
#include <CallInfo.hxx>
#include <InviteMsg.hxx>
#include <AckMsg.hxx>
#include <TransferMsg.hxx>
#include <SipRoute.hxx>

#include "Contact.hxx"

///
typedef list < Sptr < Contact > >ContactList;
typedef vector < Sptr < SipRoute > >RouteList;

/**
 */
class RgCallInfo:public CallInfo
{
  public:
    ///
    RgCallInfo();
    ///
    ~RgCallInfo();

    ///
    void setOriginator(const bool yorn);
    ///
    bool isOriginator();

    ///
    void setCallee(const SipUrl url);
    ///
    SipUrl getCallee();

    ///
     Sptr < Contact > getContact();
    ///
    void setContact(Sptr < Contact > contact);

    ///
     Sptr < ContactList > getContactList();
    ///
    void addContact(const Sptr < Contact > contact);
    ///
     Sptr < Contact > findContact(const SipMsg & msg);
    ///
    void removeContact(const Sptr < Contact > contact);
    ///
    void clearContactList();
    ///
    void setCaller(const SipUrl url);
    ///
    SipUrl getCaller();

    ///
    void setLocalSdp(const Sptr < SipSdp > sdp);
    ///
     Sptr < SipSdp > getLocalSdp();
    ///
    void setRemoteSdp(const Sptr < SipSdp > sdp);
    ///
     Sptr < SipSdp > getRemoteSdp();

    ///
    void setLocal2Sdp(const Sptr < SipSdp > sdp);
    ///
     Sptr < SipSdp > getLocal2Sdp();
    ///
    void setRemote2Sdp(const Sptr < SipSdp > sdp);
    ///
     Sptr < SipSdp > getRemote2Sdp();

    ///
    void setRingInvite(Sptr < InviteMsg > invite);
    ///
     Sptr < InviteMsg > getRingInvite();
    ///
    void setRing2Invite(Sptr < InviteMsg > invite);
    ///
     Sptr < InviteMsg > getRing2Invite();
    ///
    void setCallerRoute1List(SipRecordRouteList recordrouteList);
    ///
    void setCalleeRoute1List(SipRecordRouteList recordrouteList);
    ///
    void addRoute1(const Sptr < SipRoute > sipRoute);
    ///
     vector < SipRoute * >getRoute1List();
    ///
    void setCallerRoute2List(SipRecordRouteList recordrouteList);
    ///
    void setCalleeRoute2List(SipRecordRouteList recordrouteList);
    ///
    void addRoute2(const Sptr < SipRoute > sipRoute);
    ///
     vector < SipRoute * >getRoute2List();

    ///
    void setXferMsg(Sptr < TransferMsg > transfer);
    ///
     Sptr < TransferMsg > getXferMsg();
    ///
    void setAckMsg(Sptr < AckMsg > ack);
    ///
     Sptr < AckMsg > getAckMsg();

  protected:

  private:
    ///
     bool originator;
    ///
    SipUrl callee;
    ///
     Sptr < Contact > currentContact;
    ///
     Sptr < ContactList > contactList;
    ///
    SipUrl caller;
    ///
     Sptr < SipSdp > localSdp;
    ///
     Sptr < SipSdp > remoteSdp;
    ///
     Sptr < SipSdp > local2Sdp;
    ///
     Sptr < SipSdp > remote2Sdp;
    ///
     Sptr < InviteMsg > ringInvite;
    ///
     Sptr < InviteMsg > ring2Invite;
    ///
     Sptr < RouteList > route1List;
    ///
     Sptr < RouteList > route2List;
    ///
     Sptr < TransferMsg > xferMsg;
    ///
     Sptr < AckMsg > ackMsg;
};

#endif
