
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgCommandLine_cxx_Version =
    "$Id: RgCommandLine.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include "global.h"
#if defined(linux)
#include "getopt.h"
#else
#include "getopt_long.h"
#endif

#include <cstdio>
#include "parse3tuple.h"
#include "RgCommandLine.hxx"
#include "RgConfiguration.hxx"

///
Sptr < RgCommandLine > RgCommandLine::commandLine = 0;

///
RgCommandLine::RgCommandLine()
{
}

///
const pair < const char *, const char *>
    RgCommandLine::cmdLineOptionString[] = {
    pair < const char *, const char *>("daemon", "0"),
    pair < const char *, const char *>("cfgfile", "rg.cfg"),
    pair < const char *, const char *>("quicknet", "0"),
    pair < const char *, const char *>("retransmit", "0"),
    pair < const char *, const char *>("loglevel", "4"),
    pair < const char *, const char *>("help", "0"),
    pair < const char *, const char *>("voicemail", "0"),
    pair < const char *, const char *>("soundcard", "0"),
    pair < const char *, const char *>("retransinitial", "500"),
    pair < const char *, const char *>("retransmax", "4000"),
};

///
void
RgCommandLine::instance(const int argc, const char **argv, const char *applicationUsage)
{
    assert(commandLine == 0);

    commandLine = new RgCommandLine();
    assert(commandLine != 0);

    commandLine->setDefaultValues();
    commandLine->parseCommandLine(argc, argv, applicationUsage);
}

///
void
RgCommandLine::setDefaultValues()
{
    for (unsigned int i = 0;
         i < sizeof (cmdLineOptionString) / sizeof (cmdLineOptionString[0]); i++)
    {
        cmdLineOptionTable.insert(pair < string,
                                  string > (RgCommandLine::cmdLineOptionString[i].first,
                                            RgCommandLine::cmdLineOptionString[i].
                                            second));
    }
}

static const char *LogLevelStrings[] = {
    "LOG_EMERG",                /* system is unusable */
    "LOG_ALERT",                /* action must be taken immediately */
    "LOG_CRIT",                 /* critical conditions */
    "LOG_ERR",                  /* error conditions */
    "LOG_WARNING",              /* warning conditions */
    "LOG_NOTICE",               /* normal but significant condition */
    "LOG_INFO",                 /* informational */
    "LOG_DEBUG",                /* debug-level messages */
    "LOG_DEBUG_STACK"           /* stack debug-level messages */
};

///
void
RgCommandLine::parseCommandLine(const int argc,
                                const char **argv, const char *applicationUsage)
{
    static struct option long_options[] = {
        {RgCommandLine::cmdLineOptionString[0].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[1].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[2].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[3].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[4].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[5].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[6].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[7].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[8].first, 1, 0, 0},
        {RgCommandLine::cmdLineOptionString[9].first, 1, 0, 0},
    };

    int c = 0;
    while ((c = getopt(argc, const_cast < char **>(argv), "R:M:df:hqsmrv::")) != EOF)
    {
        switch (c)
        {
            case 'd':                  // daemon flag
                cmdLineOptionTable[long_options[0].name] = "1";
                break;

            case 'f':                  // cfgfile
                cmdLineOptionTable[long_options[1].name] = optarg;
                break;

            case 'h':                  // help
                cerr << "Usage: " << argv[0] << " " << applicationUsage << endl;
                exit(0);
                break;

            case 'q':                  // quicknet
                // check to make sure that the -m and -s option has NOT been set
                if (cmdLineOptionTable[long_options[6].name] == "1" ||
                    cmdLineOptionTable[long_options[7].name] == "1")
                {
                    cpLog(LOG_ERR, "Invalid options: [q|s|m] are mutually exclusive.");
                    exit(-1);
                }
                else
                {
                    cmdLineOptionTable[long_options[2].name] = "1";
                }
                break;

            case 's':                  // soundcard
                // check to make sure that the -q and -m  option has NOT been set
                if (cmdLineOptionTable[long_options[2].name] == "1" ||
                    cmdLineOptionTable[long_options[6].name] == "1")
                {
                    cpLog(LOG_ERR, "Soundcard not supported");
                }
                break;

            case 'm':                  // voicemail
                // check to make sure that the -q and -s option has NOT been set
                if (cmdLineOptionTable[long_options[2].name] == "1" ||
                    cmdLineOptionTable[long_options[7].name] == "1")
                {
                    cpLog(LOG_ERR, "Voicemail not supported");
                }
                break;

            case 'r':
                // retransmit
                cmdLineOptionTable[long_options[3].name] = "1";
                break;

            case 'v':
                // log level
                if (optarg == 0)
                    cmdLineOptionTable[long_options[4].name] = "7";
                else
                {
                    unsigned int loglevel = strConst2i(optarg,
                                                       LogLevelStrings,
                                                       LOG_DEBUG + 1);

                    if (loglevel <= LOG_DEBUG_STACK)
                    {
                        char s[] = { '0' + loglevel, 0 };

                        cmdLineOptionTable[long_options[4].name] = s;
                    }
                    else
                    {
                        cpLog(LOG_ERR, "Invalid option: %s", optarg);
                        exit(-1);
                    }
                }
                break;

            case 'R':                  // get initial retransmission time
                {
                    int initial = atoi(optarg);

                    cpLog(LOG_INFO, "setting initial retransmission time to %d ms",
                          initial);
                    cmdLineOptionTable[long_options[8].name] = optarg;
                    break;
                }
            case 'M':                  // get initial retransmission time
                {
                    int max = atoi(optarg);

                    cpLog(LOG_INFO, "setting maximum retransmission time to %d ms", max);
                    cmdLineOptionTable[long_options[9].name] = optarg;
                    break;
                }
            case '?':                  // getopt_long already printed an error message
                break;

            default:
                cpLog(LOG_ERR, "getopt() returned 0%o", c);
        }
    }
    if ((argc - optind) > 0)
    {
        cpLog(LOG_ERR, "Unknown option: %s", argv[optind]);
        exit(-1);
    }
}

///
Sptr < RgCommandLine > RgCommandLine::instance()
{
    assert(commandLine != 0);

    return commandLine;
}

///
const
    string &
RgCommandLine::getStringOpt(const string & cmdOption)
{
    static string nullstring = "";

    TableIter i = cmdLineOptionTable.find(cmdOption);

    if (i != cmdLineOptionTable.end())
        return i->second;

    return nullstring;
}

///
const int
RgCommandLine::getIntOpt(const string & cmdOption)
{
    TableIter i = cmdLineOptionTable.find(cmdOption);

    int ret = 0;

    if (i != cmdLineOptionTable.end())
        ret = atoi(i->second.c_str());

    return ret;
}

///
const bool
RgCommandLine::getBoolOpt(const string & cmdOption)
{
    TableIter i = cmdLineOptionTable.find(cmdOption);

    bool ret = false;

    if (i != cmdLineOptionTable.end())
        ret = (i->second == "1");

    return ret;
}
