#ifndef UA_COMMAND_LINE_Hxx
#define UA_COMMAND_LINE_Hxx

/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

#include <map>
#include <string>
#include "Sptr.hxx"

static const char *const RgCommandLineVersion =
    "$Id: RgCommandLine.hxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

/**
 *  Command line input and runtime options
 */
static const char *const appUsage = "[-dhqr] [-v[<log-level>]] [-f <config-file>]";

class RgCommandLine
{
  public:

    ///
    static void instance(const int argc,
                         const char **argv, const char *applicationUsage = appUsage);

    ///
    static Sptr < RgCommandLine > instance();

    ///
    const string & getStringOpt(const string & cmdLineOption);

    ///
    const int getIntOpt(const string & cmdLineOption);

    ///
    const bool getBoolOpt(const string & cmdLineOption);

  private:

    ///
     RgCommandLine();

    ///
    void setDefaultValues();

    ///
    void parseCommandLine(const int argc,
                          const char **argv, const char *applicationUsage);

    ///
    static Sptr < RgCommandLine > commandLine;

    ///
    typedef map < string, string > Table;

    ///
    typedef Table::iterator TableIter;

    ///
    static const pair < const char *, const char *>cmdLineOptionString[];

    ///
    Table cmdLineOptionTable;
};

#endif
