
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgConfiguration_cxx_Version =
    "$Id: RgConfiguration.cxx,v 1.4 2002/04/05 08:07:42 manish Exp $";
#include <cassert>
#include <cstdio>

#include <cpLog.h>
#include <parse3tuple.h>

#include "RgConfiguration.hxx"

///
RgConfiguration *
    RgConfiguration::rgConfiguration =
    0;
const string
    RgConfiguration::DefaultRtpPort =
    "23456";

IXJ_FILTER_FREQ_MapType
    RgConfiguration::IXJ_FILTER_FREQ_Map;

///
RgConfiguration *
RgConfiguration::instance()
{
    assert(rgConfiguration != 0);
    return rgConfiguration;
}

///
typedef enum
{
    UserNameTag = 0,
    DisplayNameTag,
    PasswordTag,
    LocalSipPortTag,
    MinRtpPortTag,
    MaxRtpPortTag,
    LocalRtpPortTag,
    SipTransportTag,
    ProxyServerTag,
    DialToneFreqTag,
    DialToneCadenceOn1Tag,
    DialToneCadenceOff1Tag,
    DialToneCadenceOn2Tag,
    DialToneCadenceOff2Tag,
    DialToneCadenceOn3Tag,
    DialToneCadenceOff3Tag,
    RingbackToneCadenceOn1Tag,
    RingbackToneCadenceOff1Tag,
    RingbackToneCadenceOn2Tag,
    RingbackToneCadenceOff2Tag,
    RingbackToneCadenceOn3Tag,
    RingbackToneCadenceOff3Tag,
    BusyToneCadenceOn1Tag,
    BusyToneCadenceOff1Tag,
    BusyToneCadenceOn2Tag,
    BusyToneCadenceOff2Tag,
    BusyToneCadenceOn3Tag,
    BusyToneCadenceOff3Tag,
    RingbackToneFreqTag,
    BusyToneFreqTag,
    NetworkRtpRateTag,
    InitialDigitTimeoutTag,
    InterDigitTimeoutTag,
    DialPatternTag,
    SpeedDialTag,
    DeviceNameTag,
    CallIdOnTag,
    CallIdTag,
    RgTimeoutTag,
    RelTimeoutTag,
    CadenceTimeoutTag,
    RgConfigurationTagMax
}
RgConfigurationTags;

///
static const char *RgConfigurationTagString[] = {
    "User_Name",
    "Display_Name",
    "Pass_Word",
    "Local_SIP_Port",
    "Min_RTP_Port",
    "Max_RTP_Port",
    "Local_RTP_Port",
    "SIP_Transport",
    "Proxy_Server",
    "Dial_Tone_Freq",
    "Dial_Tone_Cadence_On1",
    "Dial_Tone_Cadence_Off1",
    "Dial_Tone_Cadence_On2",
    "Dial_Tone_Cadence_Off2",
    "Dial_Tone_Cadence_On3",
    "Dial_Tone_Cadence_Off3",
    "Ringback_Tone_Cadence_On1",
    "Ringback_Tone_Cadence_Off1",
    "Ringback_Tone_Cadence_On2",
    "Ringback_Tone_Cadence_Off2",
    "Ringback_Tone_Cadence_On3",
    "Ringback_Tone_Cadence_Off3",
    "Busy_Tone_Cadence_On1",
    "Busy_Tone_Cadence_Off1",
    "Busy_Tone_Cadence_On2",
    "Busy_Tone_Cadence_Off2",
    "Busy_Tone_Cadence_On3",
    "Busy_Tone_Cadence_Off3",
    "Ringback_Tone_Freq",
    "Busy_Tone_Freq",
    "Network_RTP_Rate",
    "Initial_Digit_Timeout",
    "Inter_Digit_Timeout",
    "Dial_Pattern",
    "Speed_Dial",
    "Device_Name",
    "CallId_On",
    "CallId",
    "Rg_Timeout",
    "Rel_Timeout",
    "Cadence_Timeout",
    "Unknown"
};

///
void
RgConfiguration::parseCfgFileCallBack(char *tag, char *type, char *value)
{
    switch (strConst2i(tag, RgConfigurationTagString, RgConfigurationTagMax))
    {
        case UserNameTag:
            {
                RgConfiguration::instance()->setUserName(value);
            }
            break;

        case DisplayNameTag:
            {
                RgConfiguration::instance()->setDisplayName(value);
            }
            break;

        case PasswordTag:
            {
                RgConfiguration::instance()->setPassword(value);
            }
            break;

        case LocalSipPortTag:
            {
                RgConfiguration::instance()->setLocalSipPort(value);
            }
            break;

        case MinRtpPortTag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setMinRtpPort(i);
            }
            break;

        case MaxRtpPortTag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setMaxRtpPort(i);
            }
            break;

        case LocalRtpPortTag:
            {
                RgConfiguration::instance()->setLocalRtpPort(value);
            }
            break;

        case SipTransportTag:
            {
                RgConfiguration::instance()->setSipTransport(value);
            }
            break;

        case ProxyServerTag:
            {
                RgConfiguration::instance()->setProxyServer(value);
            }
            break;

        case DialToneFreqTag:
            {
                RgConfiguration::instance()->setDialToneFreq(IXJ_FILTER_FREQ_Map[value]);
            }
            break;

        case DialToneCadenceOn1Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setDialToneCadenceOn1(i);
            }
            break;

        case DialToneCadenceOff1Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setDialToneCadenceOff1(i);
            }
            break;

        case DialToneCadenceOn2Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setDialToneCadenceOn2(i);
            }
            break;

        case DialToneCadenceOff2Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setDialToneCadenceOff2(i);
            }
            break;

        case DialToneCadenceOn3Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setDialToneCadenceOn3(i);
            }
            break;

        case DialToneCadenceOff3Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setDialToneCadenceOff3(i);
            }
            break;

        case RingbackToneCadenceOn1Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setRingbackToneCadenceOn1(i);
            }
            break;

        case RingbackToneCadenceOff1Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setRingbackToneCadenceOff1(i);
            }
            break;

        case RingbackToneCadenceOn2Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setRingbackToneCadenceOn2(i);
            }
            break;

        case RingbackToneCadenceOff2Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setRingbackToneCadenceOff2(i);
            }
            break;

        case RingbackToneCadenceOn3Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setRingbackToneCadenceOn3(i);
            }
            break;

        case RingbackToneCadenceOff3Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setRingbackToneCadenceOff3(i);
            }
            break;

        case BusyToneCadenceOn1Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setBusyToneCadenceOn1(i);
            }
            break;

        case BusyToneCadenceOff1Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setBusyToneCadenceOff1(i);
            }
            break;

        case BusyToneCadenceOn2Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setBusyToneCadenceOn2(i);
            }
            break;

        case BusyToneCadenceOff2Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setBusyToneCadenceOff2(i);
            }
            break;

        case BusyToneCadenceOn3Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setBusyToneCadenceOn3(i);
            }
            break;

        case BusyToneCadenceOff3Tag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setBusyToneCadenceOff3(i);
            }
            break;

        case RingbackToneFreqTag:
            {
                RgConfiguration::instance()->
                    setRingbackToneFreq(IXJ_FILTER_FREQ_Map[value]);
            }
            break;

        case BusyToneFreqTag:
            {
                RgConfiguration::instance()->setBusyToneFreq(IXJ_FILTER_FREQ_Map[value]);
            }
            break;

        case NetworkRtpRateTag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setNetworkRtpRate(i);
            }
            break;

        case InitialDigitTimeoutTag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setInitialDigitTimeout(i);
            }
            break;

        case InterDigitTimeoutTag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setInterDigitTimeout(i);
            }
            break;

        case DialPatternTag:
            {
                int type;
                char pattern[VALUE_MAX_LENGTH];

                sscanf(value, "%d %s", &type, pattern);
                RgConfiguration::instance()->addDialPattern((DialMethodsType) type,
                                                            pattern);
            }
            break;

        case SpeedDialTag:
            {
                char key[VALUE_MAX_LENGTH];
                char dest[VALUE_MAX_LENGTH];

                sscanf(value, "%s %s", key, dest);
                RgConfiguration::instance()->addSpeedDial(key, dest);
            }
            break;

        case DeviceNameTag:
            {
                RgConfiguration::instance()->setDeviceName(value);
            }
            break;

        case CallIdOnTag:
            {
                RgConfiguration::instance()->setCallIdOn((string(value) == "True"));
            }
            break;

        case CallIdTag:
            {
                RgConfiguration::instance()->setCallId(value);
            }
            break;

        case RgTimeoutTag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setRgTimeout(i);
            }
            break;

        case RelTimeoutTag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setRelTimeout(i);
            }
            break;

        case CadenceTimeoutTag:
            {
                int i;

                sscanf(value, "%d", &i);
                RgConfiguration::instance()->setCadenceTimeout(i);
            }
            break;

        default:
            {
                cpLog(LOG_WARNING, "Unknown Tag: %s", tag);
            }
    }                                   // End of switch 
}

///
void
RgConfiguration::instance(const string cfgFile)
{
    assert(rgConfiguration == 0);

    rgConfiguration = new RgConfiguration;
    assert(rgConfiguration != 0);

    assert(parse3tuple(cfgFile.c_str(), parseCfgFileCallBack));

    rgConfiguration->show();
}

///
void
RgConfiguration::show()
{
    cpLog(LOG_INFO, "SIP Residential Gateway Configuration");
    cpLog(LOG_INFO, "--- Device ---");
    cpLog(LOG_INFO, "       Device Name : \"%s\"", getDeviceName().c_str());
    cpLog(LOG_INFO, "--- User ---");
    cpLog(LOG_INFO, "              Name : \"%s\"", getUserName().c_str());
    cpLog(LOG_INFO, "      Display Name : \"%s\"", getDisplayName().c_str());
    cpLog(LOG_INFO, "         Pass Word : \"%s\"", getPassword().c_str());

    cpLog(LOG_INFO, "--- SIP ---");
    cpLog(LOG_INFO, "              Port : \"%s\"", getLocalSipPort().c_str());
    cpLog(LOG_INFO, "         Transport : \"%s\"", getSipTransport().c_str());
    cpLog(LOG_INFO, "      Proxy Server : \"%s\"", getProxyServer().c_str());
    cpLog(LOG_INFO, "      DialTone Freq: \"%d\"", (int)getDialToneFreq());

    cpLog(LOG_INFO, "--- Dialing ---");
    cpLog(LOG_INFO, "      Dial Timeout : %d ms", getInitialDigitTimeout());
    cpLog(LOG_INFO, "Interdigit Timeout : %d ms", getInterDigitTimeout());

    cpLog(LOG_INFO, "--- RTP ---");
    cpLog(LOG_INFO, "          RTP Rate : %d ms", getNetworkRtpRate());
    cpLog(LOG_INFO, "          RTP Port : \"%s\"", getLocalRtpPort().c_str());
    cpLog(LOG_INFO, "--- Misc Features ---");
    cpLog(LOG_INFO, "          Rg  Timeout: %d s", getRgTimeout());
    cpLog(LOG_INFO, "         Rel Timeout: %d s", getRelTimeout());
    cpLog(LOG_INFO, "    Cadence  Timeout: %d s", getCadenceTimeout());
    cpLog(LOG_INFO, "--- Dial Patterns ---");
    DialingPlanListType dp = getDialPlans();

    DialingPlanListType::iterator dpIter = dp.begin();
    while (dpIter != dp.end())
    {
        cpLog(LOG_INFO, "%18d : %s", (*dpIter)->getDialMethod(),
              (*dpIter)->getDialPattern().c_str());
        dpIter++;
    }

    cpLog(LOG_INFO, "--- Speed Dials ---");
    SpeedDialMapType sd = getSpeedDials();

    SpeedDialMapType::iterator sdIter = sd.begin();
    while (sdIter != sd.end())
    {
        cpLog(LOG_INFO, "%18s : %s", sdIter->first.c_str(), sdIter->second.c_str());
        sdIter++;
    }

    cpLog(LOG_INFO, "--- Manual CallId ---");
    cpLog(LOG_INFO, "      Manual CallId: \"%s\"", (getCallIdOn()? "True" : "False"));
    cpLog(LOG_INFO, "             CallId: \"%s\"", getCallId().c_str());
    cpLog(LOG_INFO, "\n\n");

}

///
string RgConfiguration::getUserName()
{
    return userName;
}

///
string RgConfiguration::getDisplayName()
{
    return displayName;
}

///
string RgConfiguration::getPassword()
{
    return password;
}

///
string RgConfiguration::getLocalSipPort()
{
    return localSipPort;
}

///
int
RgConfiguration::getMinRtpPort()
{
    return minRtpPort;
}

///
int
RgConfiguration::getMaxRtpPort()
{
    return maxRtpPort;
}

///
string RgConfiguration::getLocalRtpPort()
{
    return localRtpPort;
}

///
string RgConfiguration::getSipTransport()
{
    return sipTransport;
}

///
string RgConfiguration::getProxyServer()
{
    return proxyServer;
}

///
IXJ_FILTER_FREQ RgConfiguration::getDialToneFreq()
{
    return dialToneFreq;
}

///
int
RgConfiguration::getDialToneCadenceOn1()
{
    return dialToneCadenceOn1;
}

///
int
RgConfiguration::getDialToneCadenceOff1()
{
    return dialToneCadenceOff1;
}

///
int
RgConfiguration::getDialToneCadenceOn2()
{
    return dialToneCadenceOn2;
}

///
int
RgConfiguration::getDialToneCadenceOff2()
{
    return dialToneCadenceOff2;
}

///
int
RgConfiguration::getDialToneCadenceOn3()
{
    return dialToneCadenceOn3;
}

///
int
RgConfiguration::getDialToneCadenceOff3()
{
    return dialToneCadenceOff3;
}

///
int
RgConfiguration::getRingbackToneCadenceOn1()
{
    return ringbackToneCadenceOn1;
}

///
int
RgConfiguration::getRingbackToneCadenceOff1()
{
    return ringbackToneCadenceOff1;
}

///
int
RgConfiguration::getRingbackToneCadenceOn2()
{
    return ringbackToneCadenceOn2;
}

///
int
RgConfiguration::getRingbackToneCadenceOff2()
{
    return ringbackToneCadenceOff2;
}

///
int
RgConfiguration::getRingbackToneCadenceOn3()
{
    return ringbackToneCadenceOn3;
}

///
int
RgConfiguration::getRingbackToneCadenceOff3()
{
    return ringbackToneCadenceOff3;
}

///
int
RgConfiguration::getBusyToneCadenceOn1()
{
    return busyToneCadenceOn1;
}

///
int
RgConfiguration::getBusyToneCadenceOff1()
{
    return busyToneCadenceOff1;
}

///
int
RgConfiguration::getBusyToneCadenceOn2()
{
    return busyToneCadenceOn2;
}

///
int
RgConfiguration::getBusyToneCadenceOff2()
{
    return busyToneCadenceOff2;
}

///
int
RgConfiguration::getBusyToneCadenceOn3()
{
    return busyToneCadenceOn3;
}

///
int
RgConfiguration::getBusyToneCadenceOff3()
{
    return busyToneCadenceOff3;
}

///
IXJ_FILTER_FREQ RgConfiguration::getRingbackToneFreq()
{
    return ringbackToneFreq;
}

///
IXJ_FILTER_FREQ RgConfiguration::getBusyToneFreq()
{
    return busyToneFreq;
}

///
int
RgConfiguration::getNetworkRtpRate()
{
    return networkRtpRate;
}

///
int
RgConfiguration::getInitialDigitTimeout()
{
    return initialDigitTimeout;
}

///
int
RgConfiguration::getInterDigitTimeout()
{
    return interDigitTimeout;
}

///
DialingPlanListType & RgConfiguration::getDialPlans()
{
    return dialPlans;
}

///
SpeedDialMapType & RgConfiguration::getSpeedDials()
{
    return speedDials;
}

///
void
RgConfiguration::setUserName(string name)
{
    userName = name;
}

///
void
RgConfiguration::setDisplayName(string name)
{
    displayName = name;
}

///
void
RgConfiguration::setPassword(string pwd)
{
    password = pwd;
}

///
void
RgConfiguration::setLocalSipPort(string port)
{
    localSipPort = port;
}

///
void
RgConfiguration::setMinRtpPort(int port)
{
    minRtpPort = port;
}                                       // RgConfiguration::getMinRtpPort

///
void
RgConfiguration::setMaxRtpPort(int port)
{
    maxRtpPort = port;
}                                       // RgConfiguration::getMaxRtpPort

///
void
RgConfiguration::setLocalRtpPort(string port)
{
    localRtpPort = port;
}

///
void
RgConfiguration::setSipTransport(string transport)
{
    sipTransport = transport;
}

///
void
RgConfiguration::setProxyServer(string server)
{
    proxyServer = server;
}

///
void
RgConfiguration::setDialToneFreq(IXJ_FILTER_FREQ freq)
{
    dialToneFreq = freq;
}

///
void
RgConfiguration::setDialToneCadenceOn1(int on1)
{
    dialToneCadenceOn1 = on1;
}

///
void
RgConfiguration::setDialToneCadenceOff1(int off1)
{
    dialToneCadenceOff1 = off1;
}

///
void
RgConfiguration::setDialToneCadenceOn2(int on2)
{
    dialToneCadenceOn2 = on2;
}

///
void
RgConfiguration::setDialToneCadenceOff2(int off2)
{
    dialToneCadenceOff2 = off2;
}

///
void
RgConfiguration::setDialToneCadenceOn3(int on3)
{
    dialToneCadenceOn3 = on3;
}

///
void
RgConfiguration::setDialToneCadenceOff3(int off3)
{
    dialToneCadenceOff3 = off3;
}

///
void
RgConfiguration::setRingbackToneCadenceOn1(int on1)
{
    ringbackToneCadenceOn1 = on1;
}

///
void
RgConfiguration::setRingbackToneCadenceOff1(int off1)
{
    ringbackToneCadenceOff1 = off1;
}

///
void
RgConfiguration::setRingbackToneCadenceOn2(int on2)
{
    ringbackToneCadenceOn2 = on2;
}

///
void
RgConfiguration::setRingbackToneCadenceOff2(int off2)
{
    ringbackToneCadenceOff2 = off2;
}

///
void
RgConfiguration::setRingbackToneCadenceOn3(int on3)
{
    ringbackToneCadenceOn3 = on3;
}

///
void
RgConfiguration::setRingbackToneCadenceOff3(int off3)
{
    ringbackToneCadenceOff3 = off3;
}

///
void
RgConfiguration::setBusyToneCadenceOn1(int on1)
{
    busyToneCadenceOn1 = on1;
}

///
void
RgConfiguration::setBusyToneCadenceOff1(int off1)
{
    busyToneCadenceOff1 = off1;
}

///
void
RgConfiguration::setBusyToneCadenceOn2(int on2)
{
    busyToneCadenceOn2 = on2;
}

///
void
RgConfiguration::setBusyToneCadenceOff2(int off2)
{
    busyToneCadenceOff2 = off2;
}

///
void
RgConfiguration::setBusyToneCadenceOn3(int on3)
{
    busyToneCadenceOn3 = on3;
}

///
void
RgConfiguration::setBusyToneCadenceOff3(int off3)
{
    busyToneCadenceOff3 = off3;
}

///
void
RgConfiguration::setRingbackToneFreq(IXJ_FILTER_FREQ freq)
{
    ringbackToneFreq = freq;
}

///
void
RgConfiguration::setBusyToneFreq(IXJ_FILTER_FREQ freq)
{
    busyToneFreq = freq;
}

///
void
RgConfiguration::setNetworkRtpRate(int rate)
{
    networkRtpRate = rate;
}

///
void
RgConfiguration::setInitialDigitTimeout(int timeout)
{
    initialDigitTimeout = timeout;
}

///
void
RgConfiguration::setInterDigitTimeout(int timeout)
{
    interDigitTimeout = timeout;
}

///
void
RgConfiguration::addDialPattern(DialMethodsType type, string pattern)
{
    DialingPlan *plan = new DialingPlan(pattern, type);

    dialPlans.push_back(plan);
}

///
void
RgConfiguration::addSpeedDial(string key, string dest)
{
    speedDials[key] = dest;
}

///
string RgConfiguration::getDeviceName() const
{
    return deviceName;
}

///
bool RgConfiguration::getCallIdOn()
{
    return myCallIdOn;
}

///
string RgConfiguration::getCallId()
{
    return myCallId;
}

///
int
RgConfiguration::getRgTimeout() const
{
    return rgTimeout;
}

///
int
RgConfiguration::getRelTimeout() const
{
    return relTimeout;
}

///
int
RgConfiguration::getCadenceTimeout() const
{
    return cadenceTimeout;
}

///
void
RgConfiguration::setCallId(const string callid)
{
    if (callid.length() <= 0)
    {
        myCallId = "";
    }
    else
    {
        myCallId = callid;
    }
}

///
void
RgConfiguration::setCallIdOn(const bool callIdOn)
{
    myCallIdOn = callIdOn;
}

///
void
RgConfiguration::setDeviceName(const string devicename)
{
    deviceName = devicename;
}

///
void
RgConfiguration::setRgTimeout(const int timeout)
{
    rgTimeout = timeout;
}

///
void
RgConfiguration::setRelTimeout(const int timeout)
{
    relTimeout = timeout;
}

///
void
RgConfiguration::setCadenceTimeout(const int timeout)
{
    cadenceTimeout = timeout;
}

//
RgConfiguration::RgConfiguration():deviceName("/dev/phone0"), password("\0"), proxyServer("\0"), dialToneFreq(f400), dialToneCadenceOn1(200), dialToneCadenceOff1(0), dialToneCadenceOn2(0), dialToneCadenceOff2(0), dialToneCadenceOn3(0), dialToneCadenceOff3(0), ringbackToneCadenceOn1(375), ringbackToneCadenceOff1(250), ringbackToneCadenceOn2(375), ringbackToneCadenceOff2(250), ringbackToneCadenceOn3(0), ringbackToneCadenceOff3(2000), busyToneCadenceOn1(750), busyToneCadenceOff1(750), busyToneCadenceOn2(0), busyToneCadenceOff2(0), busyToneCadenceOn3(0), busyToneCadenceOff3(0), ringbackToneFreq(f400), busyToneFreq(f400), localSipPort("5060"), minRtpPort(1024), maxRtpPort(2 << 16 - 1), sipTransport("UDP"), localRtpPort(RgConfiguration::DefaultRtpPort), networkRtpRate(20),
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        // 20 ms
    rgTimeout(5), relTimeout(60), myCallIdOn(false), interDigitTimeout(8000),   // 8 seconds
initialDigitTimeout(16000)              // 16 seconds
{
    IXJ_FILTER_FREQ_Map["f300_640"] = f300_640;
    IXJ_FILTER_FREQ_Map["f300_500"] = f300_500;
    IXJ_FILTER_FREQ_Map["f1100"] = f1100;
    IXJ_FILTER_FREQ_Map["f350"] = f350;
    IXJ_FILTER_FREQ_Map["f400"] = f400;
    IXJ_FILTER_FREQ_Map["f480"] = f480;
    IXJ_FILTER_FREQ_Map["f440"] = f440;
    IXJ_FILTER_FREQ_Map["f620"] = f620;
    IXJ_FILTER_FREQ_Map["f20_50"] = f20_50;
    IXJ_FILTER_FREQ_Map["f133_200"] = f133_200;
    IXJ_FILTER_FREQ_Map["f300"] = f300;
    IXJ_FILTER_FREQ_Map["f300_420"] = f300_420;
    IXJ_FILTER_FREQ_Map["f330"] = f330;
    IXJ_FILTER_FREQ_Map["f300_425"] = f300_425;
    IXJ_FILTER_FREQ_Map["f330_440"] = f330_440;
    IXJ_FILTER_FREQ_Map["f340"] = f340;
    IXJ_FILTER_FREQ_Map["f350_400"] = f350_400;
    IXJ_FILTER_FREQ_Map["f350_440"] = f350_440;
    IXJ_FILTER_FREQ_Map["f350_450"] = f350_450;
    IXJ_FILTER_FREQ_Map["f360"] = f360;
    IXJ_FILTER_FREQ_Map["f380_420"] = f380_420;
    IXJ_FILTER_FREQ_Map["f392"] = f392;
    IXJ_FILTER_FREQ_Map["f400_425"] = f400_425;
    IXJ_FILTER_FREQ_Map["f400_440"] = f400_440;
    IXJ_FILTER_FREQ_Map["f400_450"] = f400_450;
    IXJ_FILTER_FREQ_Map["f420"] = f420;
    IXJ_FILTER_FREQ_Map["f425"] = f425;
    IXJ_FILTER_FREQ_Map["f425_450"] = f425_450;
    IXJ_FILTER_FREQ_Map["f425_475"] = f425_475;
    IXJ_FILTER_FREQ_Map["f435"] = f435;
    IXJ_FILTER_FREQ_Map["f440_450"] = f440_450;
    IXJ_FILTER_FREQ_Map["f440_480"] = f440_480;
    IXJ_FILTER_FREQ_Map["f445"] = f445;
    IXJ_FILTER_FREQ_Map["f450"] = f450;
    IXJ_FILTER_FREQ_Map["f452"] = f452;
    IXJ_FILTER_FREQ_Map["f475"] = f475;
    IXJ_FILTER_FREQ_Map["f480_620"] = f480_620;
    IXJ_FILTER_FREQ_Map["f494"] = f494;
    IXJ_FILTER_FREQ_Map["f500"] = f500;
    IXJ_FILTER_FREQ_Map["f520"] = f520;
    IXJ_FILTER_FREQ_Map["f523"] = f523;
    IXJ_FILTER_FREQ_Map["f525"] = f525;
    IXJ_FILTER_FREQ_Map["f540_660"] = f540_660;
    IXJ_FILTER_FREQ_Map["f587"] = f587;
    IXJ_FILTER_FREQ_Map["f590"] = f590;
    IXJ_FILTER_FREQ_Map["f600"] = f600;
    IXJ_FILTER_FREQ_Map["f660"] = f660;
    IXJ_FILTER_FREQ_Map["f700"] = f700;
    IXJ_FILTER_FREQ_Map["f740"] = f740;
    IXJ_FILTER_FREQ_Map["f750"] = f750;
    IXJ_FILTER_FREQ_Map["f750_1450"] = f750_1450;
    IXJ_FILTER_FREQ_Map["f770"] = f770;
    IXJ_FILTER_FREQ_Map["f800"] = f800;
    IXJ_FILTER_FREQ_Map["f816"] = f816;
    IXJ_FILTER_FREQ_Map["f850"] = f850;
    IXJ_FILTER_FREQ_Map["f857_1645"] = f857_1645;
    IXJ_FILTER_FREQ_Map["f900"] = f900;
    IXJ_FILTER_FREQ_Map["f900_1300"] = f900_1300;
    IXJ_FILTER_FREQ_Map["f935_1215"] = f935_1215;
    IXJ_FILTER_FREQ_Map["f941_1477"] = f941_1477;
    IXJ_FILTER_FREQ_Map["f942"] = f942;
    IXJ_FILTER_FREQ_Map["f950"] = f950;
    IXJ_FILTER_FREQ_Map["f950_1400"] = f950_1400;
    IXJ_FILTER_FREQ_Map["f975"] = f975;
    IXJ_FILTER_FREQ_Map["f1000"] = f1000;
    IXJ_FILTER_FREQ_Map["f1020"] = f1020;
    IXJ_FILTER_FREQ_Map["f1050"] = f1050;
    IXJ_FILTER_FREQ_Map["f1100_1750"] = f1100_1750;
    IXJ_FILTER_FREQ_Map["f1140"] = f1140;
    IXJ_FILTER_FREQ_Map["f1200"] = f1200;
    IXJ_FILTER_FREQ_Map["f1209"] = f1209;
    IXJ_FILTER_FREQ_Map["f1330"] = f1330;
    IXJ_FILTER_FREQ_Map["f1336"] = f1336;
    IXJ_FILTER_FREQ_Map["lf1366"] = lf1366;
    IXJ_FILTER_FREQ_Map["f1380"] = f1380;
    IXJ_FILTER_FREQ_Map["f1400"] = f1400;
    IXJ_FILTER_FREQ_Map["f1477"] = f1477;
    IXJ_FILTER_FREQ_Map["f1600"] = f1600;
    IXJ_FILTER_FREQ_Map["f1633_1638"] = f1633_1638;
    IXJ_FILTER_FREQ_Map["f1800"] = f1800;
    IXJ_FILTER_FREQ_Map["f1860"] = f1860;
}

///
RgConfiguration::~RgConfiguration()
{
}
