#ifndef UA_CONFIGURATION_Hxx
#define UA_CONFIGURATION_Hxx

/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgConfigurationVersion =
    "$Id: RgConfiguration.hxx,v 1.5 2002/04/05 08:07:42 manish Exp $";

#include <string>

#include "DigitCollector.hxx"

#ifndef IXJUSER_H
#define IXJUSER_H
#include <linux/ixjuser.h>
#endif

/*
typedef enum {
    XferOff = 0,
    XferOn = 1,
    ConfOn = 2
} RgXferMode;
*/

typedef map < string, IXJ_FILTER_FREQ > IXJ_FILTER_FREQ_MapType;

/**
 *  SIP Residential Gateway configuration
 */
class RgConfiguration
{
  public:
    ///
    static const string DefaultRtpPort;
    ///
    static RgConfiguration *instance();
    ///
    static void instance(const string cfgFile);
    ///
    static void parseCfgFileCallBack(char *tag, char *type, char *value);
    ///
    void show();

    ///
    string getUserName();
    ///
    string getPassword();
    ///
    string getDisplayName();
    ///
    string getLocalSipPort();
    ///
    int getMinRtpPort();
    ///
    int getMaxRtpPort();
    ///
    string getLocalRtpPort();
    ///
    string getSipTransport();
    ///
    string getProxyServer();
    ///
    IXJ_FILTER_FREQ getDialToneFreq();
    /**
     */
    int getDialToneCadenceOn1();
    int getDialToneCadenceOff1();
    int getDialToneCadenceOn2();
    int getDialToneCadenceOff2();
    int getDialToneCadenceOn3();
    int getDialToneCadenceOff3();
    /**
     */
    int getRingbackToneCadenceOn1();
    int getRingbackToneCadenceOff1();
    int getRingbackToneCadenceOn2();
    int getRingbackToneCadenceOff2();
    int getRingbackToneCadenceOn3();
    int getRingbackToneCadenceOff3();
    /**
     */
    int getBusyToneCadenceOn1();
    int getBusyToneCadenceOff1();
    int getBusyToneCadenceOn2();
    int getBusyToneCadenceOff2();
    int getBusyToneCadenceOn3();
    int getBusyToneCadenceOff3();
    ///
    IXJ_FILTER_FREQ getRingbackToneFreq();
    ///
    IXJ_FILTER_FREQ getBusyToneFreq();
    ///
    int getNetworkRtpRate();
    ///
    int getInitialDigitTimeout();
    ///
    int getInterDigitTimeout();
    ///
     DialingPlanListType & getDialPlans();
    ///
     SpeedDialMapType & getSpeedDials();
    ///
    void setUserName(string name);
    ///
    void setPassword(string pwd);
    ///
    void setDisplayName(string name);
    ///
    void setLocalSipPort(string port);
    ///
    void setMinRtpPort(int port);
    ///
    void setMaxRtpPort(int port);
    ///
    void setLocalRtpPort(string port);
    ///
    void setSipTransport(string transport);
    ///
    void setProxyServer(string server);
    ///
    void setDialToneFreq(IXJ_FILTER_FREQ freq);
    /**
     */
    void setDialToneCadenceOn1(int on1);
    void setDialToneCadenceOff1(int off1);
    void setDialToneCadenceOn2(int on2);
    void setDialToneCadenceOff2(int off2);
    void setDialToneCadenceOn3(int on3);
    void setDialToneCadenceOff3(int off3);

    /**
     */
    void setRingbackToneCadenceOn1(int on1);
    void setRingbackToneCadenceOff1(int off1);
    void setRingbackToneCadenceOn2(int on2);
    void setRingbackToneCadenceOff2(int off2);
    void setRingbackToneCadenceOn3(int on3);
    void setRingbackToneCadenceOff3(int off3);

    /**
     */
    void setBusyToneCadenceOn1(int on1);
    void setBusyToneCadenceOff1(int off1);
    void setBusyToneCadenceOn2(int on2);
    void setBusyToneCadenceOff2(int off2);
    void setBusyToneCadenceOn3(int on3);
    void setBusyToneCadenceOff3(int off3);

    ///
    void setRingbackToneFreq(IXJ_FILTER_FREQ freq);
    ///
    void setBusyToneFreq(IXJ_FILTER_FREQ freq);
    ///
    void setNetworkRtpRate(int rate);
    ///
    void setInitialDigitTimeout(int timeout);
    ///
    void setInterDigitTimeout(int timeout);
    ///
    void addDialPattern(DialMethodsType type, string pattern);
    ///
    void addSpeedDial(string key, string dest);
    ///
    string getDeviceName() const;
    ///
    void setDeviceName(const string devicename);
    ///
    void setCallIdOn(const bool callIdOn);
    ///
    bool getCallIdOn();
    ///
    void setCallId(const string callid);
    ///
    string getCallId();
    ///
    void setRgTimeout(const int timeout);
    ///
    int getRgTimeout() const;

    ///
    void setRelTimeout(const int timeout);
    ///
    int getRelTimeout() const;
    ///
    void setCadenceTimeout(const int timeout);
    ///
    int getCadenceTimeout() const;

    ///
    ~RgConfiguration();

  protected:
    ///

  private:
    ///
     RgConfiguration();

    ///
    string userName;
    ///
    string displayName;
    ///
    string password;
    ///
    string localSipPort;
    ///
    int minRtpPort;
    ///
    int maxRtpPort;
    ///
    string localRtpPort;
    ///
    string sipTransport;
    ///
    string proxyServer;
    ///
    string gwServer;
    ///
    static IXJ_FILTER_FREQ_MapType IXJ_FILTER_FREQ_Map;
    ///
    IXJ_FILTER_FREQ dialToneFreq;
    ///
    int dialToneCadenceOn1;
    int dialToneCadenceOff1;
    int dialToneCadenceOn2;
    int dialToneCadenceOff2;
    int dialToneCadenceOn3;
    int dialToneCadenceOff3;
    ///
    int ringbackToneCadenceOn1;
    int ringbackToneCadenceOff1;
    int ringbackToneCadenceOn2;
    int ringbackToneCadenceOff2;
    int ringbackToneCadenceOn3;
    int ringbackToneCadenceOff3;
    ///
    int busyToneCadenceOn1;
    int busyToneCadenceOff1;
    int busyToneCadenceOn2;
    int busyToneCadenceOff2;
    int busyToneCadenceOn3;
    int busyToneCadenceOff3;
    ///
    IXJ_FILTER_FREQ ringbackToneFreq;
    ///
    IXJ_FILTER_FREQ busyToneFreq;
    ///
    int networkRtpRate;
    ///
    int initialDigitTimeout;
    ///
    int interDigitTimeout;
    ///
    DialingPlanListType dialPlans;
    ///
    SpeedDialMapType speedDials;

    /**
     * other parameters
     */
    ///
    string deviceName;
    ///
    string myCallId;
    ///
    bool myCallIdOn;
    ///
    int rgTimeout;
    ///
    int relTimeout;
    ///
    int cadenceTimeout;
    static RgConfiguration *rgConfiguration;
};

#endif
