#ifndef RgDeviceEvent_H
#define RgDeviceEvent_H

/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgDeviceEventVersion =
    "$Id: RgDeviceEvent.hxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include "Sptr.hxx"
#include "SipProxyEvent.hxx"
#include "Fifo.h"
#include "SipCallId.hxx"

#include "SipTransceiver.hxx"

//TODO move typedef's into RgDeviceEvent
///
typedef enum
{
    DeviceSignalNULL,
    DeviceSignalDialToneStart,
    DeviceSignalDialToneStop,
    DeviceSignalRingStart,
    DeviceSignalRingStop,
    DeviceSignalRemoteRingbackStart,
    DeviceSignalRemoteRingbackStop,
    DeviceSignalLocalRingbackStart,
    DeviceSignalLocalRingbackStop,
    DeviceSignalBusyStart,
    DeviceSignalBusyStop,
    DeviceSignalFastBusyStart,
    DeviceSignalFastBusyStop,
    DeviceSignalErrorToneStart,
    DeviceSignalErrorToneStop,
    DeviceSignalDigit0,
    DeviceSignalDigit1,
    DeviceSignalDigit2,
    DeviceSignalDigit3,
    DeviceSignalDigit4,
    DeviceSignalDigit5,
    DeviceSignalDigit6,
    DeviceSignalDigit7,
    DeviceSignalDigit8,
    DeviceSignalDigit9,
    DeviceSignalDigitStar,
    DeviceSignalDigitHash,
    DeviceSignalDigitA,
    DeviceSignalDigitB,
    DeviceSignalDigitC,
    DeviceSignalDigitD,
    DeviceSignalFwding,
    DeviceSignalStartCallWaitingBeep,
    DeviceSignalStopCallWaitingBeep,

    DeviceSignalPSTNOffHook,
    DeviceSignalPSTNOnHook,
    DeviceSignalPSTNDtmf0,
    DeviceSignalPSTNDtmf1,
    DeviceSignalPSTNDtmf2,
    DeviceSignalPSTNDtmf3,
    DeviceSignalPSTNDtmf4,
    DeviceSignalPSTNDtmf5,
    DeviceSignalPSTNDtmf6,
    DeviceSignalPSTNDtmf7,
    DeviceSignalPSTNDtmf8,
    DeviceSignalPSTNDtmf9,
    DeviceSignalPSTNDtmfStar,
    DeviceSignalPSTNDtmfHash,
    DeviceSignalPSTNDtmfA,
    DeviceSignalPSTNDtmfB,
    DeviceSignalPSTNDtmfC,
    DeviceSignalPSTNDtmfD
}
DeviceSignalType;

///
typedef enum
{
    DeviceEventUndefined = 0,
    DeviceEventShutdown,
    DeviceEventHookUp,
    DeviceEventHookDown,
    DeviceEventFlash,
    DeviceEventDtmf0 = 100,
    DeviceEventDtmf1,
    DeviceEventDtmf2,
    DeviceEventDtmf3,
    DeviceEventDtmf4,
    DeviceEventDtmf5,
    DeviceEventDtmf6,
    DeviceEventDtmf7,
    DeviceEventDtmf8,
    DeviceEventDtmf9,
    DeviceEventDtmfStar,
    DeviceEventDtmfHash,
    DeviceEventCallUrl = 200,
    DeviceEventPSTNHookUp = 300,
    DeviceEventPSTNHookDown,
    DeviceEventPSTNDialtone,
    DeviceEventPSTNRingback,
    DeviceEventPSTNConnect,
    DeviceEventPSTNWink,
    DeviceEventPSTNBusy,
    DeviceEventPSTNNoAnswer,
    DeviceEventPSTNRingbackCadenceTimer,
    DeviceEventPSTNRinging,
    DeviceEventPSTNDtmfStar,
    DeviceEventPSTNDtmfHash,
    DeviceEventPSTNDtmf0 = 400,
    DeviceEventPSTNDtmf1,
    DeviceEventPSTNDtmf2,
    DeviceEventPSTNDtmf3,
    DeviceEventPSTNDtmf4,
    DeviceEventPSTNDtmf5,
    DeviceEventPSTNDtmf6,
    DeviceEventPSTNDtmf7,
    DeviceEventPSTNDtmf8,
    DeviceEventPSTNDtmf9,

}
DeviceEventType;

/** Events from a device
 *
 */
class RgDeviceEvent:public SipProxyEvent
{
  public:
    ///
    RgDeviceEvent(const Sptr < Fifo < Sptr < SipProxyEvent > > >outputFifo);
    ///
     virtual ~ RgDeviceEvent();
    ///
    virtual const char *const name() const;

    ///
    void setSipStack(const Sptr < SipTransceiver > stack);
    ///
    const Sptr < SipTransceiver > getSipStack();

    //TODO move type to protected area
    ///
    DeviceEventType type;
    ///
     Sptr < SipCallId > callId;
    ///
    Data text;

  protected:

  private:
    ///
     Sptr < SipTransceiver > sipStack;
};

#endif
