#ifndef RgHardwareEvent_H
#define RgHardwareEvent_H

/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgHardwareEventVersion =
    "$Id: RgHardwareEvent.hxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <vector>

//#include <Sptr.hxx>
//#include <Fifo.h>

#include <rtpTypes.h>
#include "RgDeviceEvent.hxx"
#include "SipProxyEvent.hxx"

///
enum HardwareMsgType
{
    HardwareSignalType,
    HardwareAudioType
};

///
enum HardwareAudioRequestType
{
    AudioStart,
    AudioStop,
    AudioSuspend,
    AudioResume
};

#define RGW_HOST_STRING_LEN (128)
#define RGW_MODE_LEN (128)

///
struct HardwareAudioRequest
{
    HardwareAudioRequestType type;
    int localPort;
    int remotePort;
    char localHost[RGW_HOST_STRING_LEN];
    char remoteHost[RGW_HOST_STRING_LEN];
    char mode[RGW_MODE_LEN];    // should this be an enum ???
    bool echoCancellation;
    int rtpPacketSize;
    bool sendRingback;
    RtpPayloadType networkFormat;
};

///
typedef union
{
    DeviceSignalType signal;
    struct HardwareAudioRequest request;
}
HardwareMsgUnion;

/// Message for hardware
class RgHardwareEvent:public SipProxyEvent
{
  public:
    ///
    RgHardwareEvent(const Sptr < Fifo < Sptr < SipProxyEvent > > >outputFifo);
    ///
     virtual ~ RgHardwareEvent();
    ///
    virtual const char *const name() const;

    //TODO move public data to protected area
    ///
    HardwareMsgType type;
    ///
    HardwareMsgUnion signalOrRequest;
    ///
     vector < string > dataList;
};

#endif
