
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: RgOperator.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include "global.h"
#include <SipEvent.hxx>

#include "RgOperator.hxx"
#include "RgTimerEvent.hxx"
#include "RgDigitTimerEvent.hxx"
#include "RgDeviceEvent.hxx"
#include "RgCallInfo.hxx"
#include "RgConfiguration.hxx"
#include "CallContainer.hxx"

///
RgOperator::RgOperator()
{
}

RgOperator::~RgOperator()
{
}

///

const char *const
RgOperator::name() const
{
    return "RgOperator";
}

///
bool RgOperator::setTimer(const Sptr < SipProxyEvent > event, unsigned int timeout,
                          const char *createdbyopr)
{
    if (timeout <= 0)
        return false;

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    Sptr < RgDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast(event);
    Sptr < RgDigitTimerEvent > digitTimerEvent;
    digitTimerEvent.dynamicCast(event);
    Sptr < RgTimerEvent > gwTimerEvent;
    gwTimerEvent.dynamicCast(event);

    if (sipEvent == 0 && deviceEvent == 0 && digitTimerEvent == 0 && gwTimerEvent == 0)
    {
        cpLog(LOG_DEBUG, "Expecting sipEvent, deviceEvent or digitTimerEvent");
        assert(0);
    }                                   /*if sipEvent  or deviceEvent */

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < Fifo < Sptr < SipProxyEvent > > >msgQ;
    Sptr < SipTransceiver > sipStack;
    Sptr < SipCallId > callId;

    if (sipEvent != 0)
    {
        msgQ = sipEvent->getFifo();
        sipStack = sipEvent->getSipStack();
        Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
        assert(sipMsg != 0);
        callId = new SipCallId(sipMsg->getCallId());
    }
    else if (deviceEvent != 0)
    {
        msgQ = deviceEvent->getFifo();
        sipStack = deviceEvent->getSipStack();
        callId = deviceEvent->callId;
    }
    else if (digitTimerEvent != 0)
    {
        msgQ = digitTimerEvent->getFifo();
        sipStack = digitTimerEvent->getSipStack();
        callId = digitTimerEvent->callId;
    }
    else if (gwTimerEvent != 0)
    {
        msgQ = gwTimerEvent->getFifo();
        sipStack = gwTimerEvent->getSipStack();
        callId = gwTimerEvent->getCallId();
    }

    Sptr < RgTimerEvent > timerEvent = new RgTimerEvent(msgQ);
    timerEvent->setCallId(callId);

    cpLog(LOG_DEBUG, "Start TIMER for %d s.", timeout);

    // SAve the timer event in  Call Info
    call->setTimer(timerEvent);
    timerEvent->setCallInfo(call, event->getCallContainer());
    // Set the sipStack
    timerEvent->setSipStack(sipStack);

    /// Set the setting Operator.
    timerEvent->setCreatedByOpr(createdbyopr);

    timerEvent->startTimer(timerEvent, timeout * 1000);

    return true;
}                                       //setTimer

///
bool RgOperator::cancelTimer(const Sptr < SipProxyEvent > event)
{
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    Sptr < RgDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast(event);
    Sptr < RgDigitTimerEvent > digitTimerEvent;
    digitTimerEvent.dynamicCast(event);

    if (sipEvent == 0 && deviceEvent == 0 && digitTimerEvent == 0)
    {
        cpLog(LOG_DEBUG, "Expecting sipEvent, deviceEvent or digitTimerEvent.");
        return false;
    }                                   /*if sipEvent  or deviceEvent */

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    if (call->getTimer() != 0)
    {
        /// Clear the setting Operator.
        Sptr < RgTimerEvent > timerEvent;
        timerEvent.dynamicCast(call->getTimer());
        timerEvent->setCreatedByOpr("");

        cpLog(LOG_DEBUG, "Cancelling timer.");
        call->getTimer()->cancelTimer();
        return true;
    }                                   //getTimer

    cpLog(LOG_ERR, "No timer was found.");
    return false;
}                                       // cancelTiemr

///
bool RgOperator::isTimeout(const Sptr < SipProxyEvent > event)
{
    Sptr < RgTimerEvent > timerEvent;
    timerEvent.dynamicCast(event);

    if (timerEvent != 0)
    {
        return true;
    }
    else
    {
        return false;
    }
}
