
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const RgStateMachine_cxx_Version =
    "$Id: RgStateMachine.cxx,v 1.3 2002/04/05 08:07:42 manish Exp $";

#include "global.h"
#include "State.hxx"
#include "RgStateMachine.hxx"
#include "RgCallInfo.hxx"

#include "RgConfiguration.hxx"

#include "gwStateIdle.hxx"
#include "gwStateIPDialing.hxx"
#include "gwStateIPTrying.hxx"
#include "gwStateInCall.hxx"
#include "gwStatePSTNDialing.hxx"
#include "gwStatePSTNTrying.hxx"
#include "gwStateBusyHere.hxx"
#include "gwStateWait.hxx"

///
RgStateMachine::RgStateMachine()
{
    cpLog(LOG_DEBUG, "RGW State Machine");

    addState(new gwStateIdle);
    addState(new gwStateIPDialing);
    addState(new gwStateIPTrying);
    addState(new gwStateInCall);
    addState(new gwStatePSTNDialing);
    addState(new gwStatePSTNTrying);
    addState(new gwStateBusyHere);
    addState(new gwStateWait);
/*
        addState( new StateBusy );
        addState( new StateCallEnded );
        addState( new StateDialing );
        addState( new StateError );
        addState( new StateFastBusy );
        addState( new StateIdle );
        addState( new StateInCall );
        addState( new StateRinging );
        addState( new StateTrying );
        addState( new StateBusyHere );
        addState( new StateOnhold );
*/

}                                       // RgStateMachine::RgStateMachine

///
RgStateMachine::~RgStateMachine()
{
}                                       // RgStateMachine::~RgStateMachine

void
RgStateMachine::process(const Sptr < SipProxyEvent > event)
{
    assert(event != 0);

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    cpLog(LOG_DEBUG, "Current State is %s", call->getState()->name());

    Sptr < State > nextState = call->getState()->process(event);

    if (nextState != PROXY_CONTINUE)
    {
        call->setState(nextState);
        cpLog(LOG_DEBUG, "Next State is %s", nextState->name());
    }
}                                       // RgStateMachine::process

///
Sptr < State > RgStateMachine::findState(const string stateName)
{
    StateIter
        iter =
        myStates.
        begin();

    while (iter != myStates.end())
    {
        if (stateName == (*iter)->name())
        {
            return *iter;
        }
        iter++;
    }
    return 0;
}

///
const char *const
RgStateMachine::name() const
{
    return "Residential Gateway State Machine";
}                                       // RgStateMachine::name
