
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpBusyHere.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include "SipEvent.hxx"
#include "InviteMsg.hxx"

#include "gwOpBusyHere.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"

///
gwOpBusyHere::gwOpBusyHere()
{
}

///
gwOpBusyHere::~gwOpBusyHere()
{
}

///
const char *const
gwOpBusyHere::name() const
{
    return "gwOpBusyHere";
}

//***************************************************************************
//      gwOpBusyHere::process
//
//      Description     :
//      This is the operator for gwStateBusyHere. 
//      (1) Receive INVITE from IP user
//      (2) Sends 486 Busy to IP user 
//
//***************************************************************************
const Sptr < State >
gwOpBusyHere::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpBusyHere operation");

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    if (sipEvent == 0)
    {
        return 0;
    }
    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert(sipMsg != 0);

    Sptr < InviteMsg > msg;
    msg.dynamicCast(sipMsg);
    if (msg == 0)
    {
        return 0;
    }

    Sptr < SipCallId > currentCall = RgDevice::instance()->getCallId();
    if (currentCall != 0)
    {
        if (msg->getCallId() == *currentCall)
        {
            // INVITE for the same call
            return 0;
        }
    }

    cout << "Busy - Cannot accept call " << msg->getFrom().encode().
        getData();

    StatusMsg status(*msg, 486 /*busy here */ );

    // Add myself to the contact
    Sptr < SipUrl > myUrl = new SipUrl;
    myUrl->setUserValue(Data(RgConfiguration::instance()->getUserName()), Data("phone"));
    myUrl->setHost(Data(theSystem.gethostAddress()));
    myUrl->setPort(atoi(RgConfiguration::instance()->getLocalSipPort().c_str()));
    SipContact me;

    me.setUrl(myUrl);
    status.setNumContact(0);            // Clear
    status.setContact(me);

    sipEvent->getSipStack()->sendReply(status);

    return 0;
}
