
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpDialError.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <AckMsg.hxx>
#include <StatusMsg.hxx>

#include "gwOpDialError.hxx"
#include "RgStateMachine.hxx"
#include "RgCallInfo.hxx"

#include "RgHardwareEvent.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"
#include "RgDeviceEvent.hxx"
#include "RgCallContainer.hxx"
#include "RgOperator.hxx"
#include "RgDigitTimerEvent.hxx"

///
gwOpDialError::gwOpDialError()
{
}

///
gwOpDialError::~gwOpDialError()
{
}

///
const char *const
gwOpDialError::name() const
{
    return "gwOpDialError";
}

//***************************************************************************
//       gwOpDialError::process
//
//       Description     :
//       This is the operator for gwStateIPDialing.
//       For a PSTN-IP call, it performs the following functions:
//       (1) Receive DigitTimer event
//       (1a) Start Release Timer
//       (2) Play Busy tone to the PSTN user    
//       (3) Disconnect from PSTN (i.e, go OnHook on PSTN port)
//       (4) Change state to gwStateWait
//
//***************************************************************************
///
const Sptr < State >
gwOpDialError::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpDialError operation");

    Sptr < RgDigitTimerEvent > timerEvent;
    timerEvent.dynamicCast(event);
    if (timerEvent == 0)
    {
        return 0;
    }

    /* Adapted for sip-1.3.0
     * In 1.0 this operator was processed only when others could not find next state.
     * In 1.3 all operators are processed, whether or not next state is found.
     * Hence, needed to add the foll patch, to remove the error msg thrown
     * by this operator when used in 1.3
     */
    //if( !( digitCollector->isValidDial() ) )
    if (RgDevice::instance()->getDigitCollector()->isValidDial())
    {
        cpLog(LOG_DEBUG, "Dialing valid");
        return 0;
    }

    cpLog(LOG_DEBUG, "Invalid number %s",
          RgDevice::instance()->getDigitCollector()->getDial().c_str());

    //Start timer for release
    Sptr < RgOperator > RgOpr;
    if (!RgOpr->
        setTimer(event, RgConfiguration::instance()->getRelTimeout(), "gwOpDialError"))
    {
        cpLog(LOG_DEBUG, "Release Timer not started");
    }
    else
    {
        cpLog(LOG_DEBUG, "Release Timer started");
    }

    //start playing busy tone.... 
    cpLog(LOG_DEBUG, "Start Busy Tone");
    cout << endl << "Start Busy Tone " << endl;

    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalBusyStart;

    RgDevice::getDeviceQueue()->add(signal);

    vusleep(5000000);                   // sleep for 5 sec.

    // Send Release signal to PSTN user
    // i.e., send On Hook event on the PSTN line
    Sptr < RgHardwareEvent > signalOnHook =
        new RgHardwareEvent(RgDevice::getDeviceQueue());

    signalOnHook->type = HardwareSignalType;
    signalOnHook->signalOrRequest.signal = DeviceSignalPSTNOnHook;

    RgDevice::getDeviceQueue()->add(signalOnHook);

    // Change state to gwStateWait
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);
    cout << "WAIT" << endl;

    return stateMachine->findState("gwStateWait");

}
