static const char *const version =
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

    "$Id: gwOpIPEndsCall.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <ByeMsg.hxx>
#include <StatusMsg.hxx>

#include "RgStateMachine.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"
#include "RgCallContainer.hxx"
#include "RgOperator.hxx"

#include "gwOpIPEndsCall.hxx"

///
gwOpIPEndsCall::gwOpIPEndsCall()
{
}

///
gwOpIPEndsCall::~gwOpIPEndsCall()
{
}

///
const char *const
gwOpIPEndsCall::name() const
{
    return "gwOpIPEndsCall";
}

//**************************************************************************
//      gwOpIPEndsCall::process
//
//      Description     : 
//      This is the operator for StateInCall. 
//      It performs the following functions:
//      (1) Receive BYE from IP user
//      (2) Send Disconnect signal to PSTN user
//      (3) Send 200 Ok to IP user
//      (4a) Change state to gwStateIdle if IP-PSTN call
//      (4b) Start Release timer & Change state to gwStateWait if PSTN-IP call
//
//**************************************************************************
const Sptr < State >
gwOpIPEndsCall::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpIPEndsCall operation");

    // Receive BYE from IP user
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    if (sipEvent == 0)
    {
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert(sipMsg != 0);

    Sptr < ByeMsg > msg;
    msg.dynamicCast(sipMsg);
    if (msg == 0)
    {
        return 0;
    }
    cpLog(LOG_DEBUG, "IP User Ends Call");

    // Send Disconnect signal to PSTN user
    Sptr < RgHardwareEvent > signalOnHook =
        new RgHardwareEvent(RgDevice::getDeviceQueue());

    signalOnHook->type = HardwareSignalType;
    signalOnHook->signalOrRequest.signal = DeviceSignalPSTNOnHook;

    RgDevice::getDeviceQueue()->add(signalOnHook);

    usleep(500);                        // 0.5 ms

    // Send 200 OK to IP user
    StatusMsg status(*msg, 200 /*OK*/);

    sipEvent->getSipStack()->sendReply(status);
    cpLog(LOG_DEBUG, "200 OK sent to user");

    // check if BYE msg is for current contact
    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < Contact > curContact;
    Sptr < Contact > byeContact;
    curContact = call->getContact();
    assert(curContact != 0);
    byeContact = call->findContact(*msg);

    if (byeContact == 0)
    {
        cpLog(LOG_DEBUG, "Bye message is not for a known contact");
        return 0;
    }

    if (*curContact != *byeContact)
    {
        cpLog(LOG_DEBUG, "Bye message is not for the current contact");
        return 0;
    }

    // Show caller information
    cout << VTime::strftime("%y/%m/%d %H:%M:%S")
        << " Call end " << msg->getFrom().encode().getData();

    if (curContact->getStatus() != 200)
    {
        cpLog(LOG_DEBUG, "Call Initiated by IP user....changing to gwStateIdle");
    }
    else
    {
        cpLog(LOG_DEBUG,
              "Call Initiated by PSTN user.Starting PSTNTimer...changing to gwStateWait");
        // Show caller information  -kle

        // Start Timer
        Sptr < RgOperator > RgOpr;
        if (!RgOpr->
            setTimer(event, RgConfiguration::instance()->getRelTimeout(),
                     "gwOpIPEndsCall"))
        {
            cpLog(LOG_DEBUG, "Release Timer not started");
        }
        else
        {
            cpLog(LOG_DEBUG, "Release Timer started");
        }

        //change state to gwStateWait

        Sptr < RgStateMachine > stateMachine;
        stateMachine.dynamicCast(event->getCallInfo()->getFeature());
        assert(stateMachine != 0);
        cout << "WAIT" << endl;

        return stateMachine->findState("gwStateWait");
    }

    // Remove call info from call container
    Sptr < RgCallContainer > calls;
    calls.dynamicCast(event->getCallContainer());
    assert(calls != 0);

    // calls->deleteCall( *(RgDevice::instance()->getCallId()) );
    SipCallId callId = call->getContact()->getInviteMsg().getCallId();

    calls->deleteCall(callId);
    // Clear call id in device
    RgDevice::instance()->setCallId(0);
    cpLog(LOG_DEBUG, "Call details deleted");

    // Change state to gwStateIdle
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);
    cout << "READY" << endl;
    cpLog(LOG_DEBUG, "state changing from gwStateInCall to gwStateIdle");
    return stateMachine->findState("gwStateIdle");

}
