static const char *const version =
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

    "$Id: gwOpIPNoAnswer.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <StatusMsg.hxx>
#include <CancelMsg.hxx>

#include "RgHardwareEvent.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgStateMachine.hxx"
#include "RgConfiguration.hxx"
#include "RgOperator.hxx"
#include "RgTimerEvent.hxx"
#include "RgDeviceEvent.hxx"
#include "RgCallContainer.hxx"

#include "gwOpIPNoAnswer.hxx"

///
gwOpIPNoAnswer::gwOpIPNoAnswer()
{
}

///
gwOpIPNoAnswer::~gwOpIPNoAnswer()
{
}

///
const char *const
gwOpIPNoAnswer::name() const
{
    return "gwOpIPNoAnswer";
}

//**************************************************************************/
//      gwOpIPNoAnswer::process
//
//      Description     : 
//      This is the operator for gwStateIPTrying. 
//      For a PSTN-IP call, it performs the following functions:
//      (1) Receive Timeout for ringing. 
//      (2) Start Busy tone to PSTN user
//      (3) Send Release signal (OnHook) to PSTN user.
//      (4) send CANCEL to IP user.
//      (5) Start timer for release.
//      (6) Change state to gwStateWait.
//**************************************************************************/
const Sptr < State >
gwOpIPNoAnswer::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpIPNoAnswer operation");

    Sptr < RgTimerEvent > timerEvent;
    timerEvent.dynamicCast(event);
    if (timerEvent == 0)
    {
        return 0;
    }

    // Catch the Timeout event
    Sptr < RgOperator > RgOpr;
    if (RgOpr->isTimeout(timerEvent) == false)
    {
        //cpLog( LOG_DEBUG, "RgOpr->isTimeout(timerEvent) == false " );
        return 0;
    }
    if (strcmp("gwOpIPRinging", timerEvent->getCreatedByOpr()) != 0)
    {
        cpLog(LOG_DEBUG, "Time out not me ");
//         cout << "Time out not me " << endl;
        return 0;
    }
    cpLog(LOG_DEBUG, "Timeout for IP ringing ");

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < Contact > contact;
    contact = call->getContact();

    if (contact->getStatus() <= 100)
    {
        cpLog(LOG_DEBUG,
              "No Response case! To be processed by ResponseTimeout, not NoAnswer.");
        return 0;
    }

    // start playing busy tone.... 
    cpLog(LOG_DEBUG, "Start Busy Tone");
    cout << "Start Busy Tone " << endl;

    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalBusyStart;

    RgDevice::getDeviceQueue()->add(signal);

    vusleep(5000000);                   // sleep for 5 sec

    // Send Release signal to PSTN user
    // i.e., send On Hook event on the PSTN line
    Sptr < RgHardwareEvent > signalOnHook =
        new RgHardwareEvent(RgDevice::getDeviceQueue());

    signalOnHook->type = HardwareSignalType;
    signalOnHook->signalOrRequest.signal = DeviceSignalPSTNOnHook;

    RgDevice::getDeviceQueue()->add(signalOnHook);

    cpLog(LOG_DEBUG, "PSTN goes ON_HOOK in IPNoAnswer Operation");

    //send the CANCEL to IP user
    CancelMsg msg(contact->getInviteMsg());     //check if Invite msg can be used...???

    timerEvent->getSipStack()->sendAsync(msg);

    //Start Timer
    if (!RgOpr->
        setTimer(event, RgConfiguration::instance()->getRelTimeout(), "gwOpIPNoAnswer"))
    {
        cpLog(LOG_DEBUG, "Release Timer not started");
    }
    else
    {
        cpLog(LOG_DEBUG, "Release Timer started");
    }

    //change state to gwStateWait
    //
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    cout << "WAIT" << endl;
    return stateMachine->findState("gwStateWait");

}
