static const char *const version =
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

    "$Id: gwOpIPRinging.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <StatusMsg.hxx>

#include "RgHardwareEvent.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgStateMachine.hxx"
#include "RgConfiguration.hxx"
#include "RgTimerEvent.hxx"
#include "RgOperator.hxx"

#include "gwOpIPRinging.hxx"

///
gwOpIPRinging::gwOpIPRinging()
{
}

///
gwOpIPRinging::~gwOpIPRinging()
{
}

///
const char *const
gwOpIPRinging::name() const
{
    return "gwOpIPRinging";
}

//**************************************************************************
//      gwOpIPRinging::process
//
//      Description     : 
//      This is the operator for gwStateIPTrying. 
//      For a PSTN-IP call, it performs the following functions:
//      (1) Receive 180 ringing from IP user
//      (2) Stop the Timer started in gwOpStartTimer. 
//      (3) Start the No Answer timer
//      (4) Send Ringback tone to the PSTN user
//
//**************************************************************************
const Sptr < State >
gwOpIPRinging::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpIPRinging operation");

    // check for the 180 (Ringback) from IP user
    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    if (sipEvent == 0)
    {
        return 0;
    }

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert(sipMsg != 0);

    Sptr < StatusMsg > msg;
    msg.dynamicCast(sipMsg);
    if (msg == 0)
    {
        return 0;
    }

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    if (*(call->findContact(*msg)) != *(call->getContact()))
    {
        cpLog(LOG_DEBUG, "Not the current call");
        return 0;
    }

    call->getContact()->update(*msg);

    int status = msg->getStatusLine().getStatusCode();

    if (status != 180)
    {
        return 0;                       // Done
    }
    cpLog(LOG_DEBUG, "received 180 from IP user\n");

    // Stop the Timer started in gwOpStartTimer. 
    Sptr < RgOperator > RgOpr;
    RgOpr->cancelTimer(event);
    cpLog(LOG_DEBUG, "Cancelling the response timer .");

    // Start the Timer
    if (!RgOpr->
        setTimer(event, RgConfiguration::instance()->getRgTimeout(), "gwOpIPRinging"))
    {
        cpLog(LOG_DEBUG, "NO Answer Timer not started");
    }
    else
    {
        cpLog(LOG_DEBUG, "NO Answer Timer started");
    }

    // start playing the Ringback tone
    cpLog(LOG_DEBUG, "Start Ringback Tone");
    cout << "Start Ringback Tone " << endl;

    RgDevice::instance()->getDigitCollector()->reset();

    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalLocalRingbackStart;

    RgDevice::getDeviceQueue()->add(signal);

    return 0;
}
