
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpInviteUrl.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include "global.h"
#include <InviteMsg.hxx>

#include "RgDigitTimerEvent.hxx"
#include "DigitCollector.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"
#include "RgCallInfo.hxx"
#include "RgStateMachine.hxx"

#include "gwOpInviteUrl.hxx"

///
gwOpInviteUrl::gwOpInviteUrl()
{
}

///
gwOpInviteUrl::~gwOpInviteUrl()
{
}

///
const char *const
gwOpInviteUrl::name() const
{
    return "gwOpInviteUrl";
}

//**************************************************************************
//      gwOpInviteUrl::process
//
//      Description     : 
//      This is the operator for gwStateIPDialing. 
//      For a PSTN->IP call, it performs the following functions:
//      (1) Construct the INVITE from IP extension & send it
//      (2) Change the state to gwStateIPTrying
//
//**************************************************************************
const Sptr < State >
gwOpInviteUrl::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpInviteUrl operation");

    Sptr < RgDigitTimerEvent > timerEvent;
    timerEvent.dynamicCast(event);
    if (timerEvent == 0)
    {
        return 0;
    }

    Sptr < DigitCollector > digitCollector = RgDevice::instance()->getDigitCollector();
    assert(digitCollector != 0);

    if (!(digitCollector->isValidDial()))
    {
        cpLog(LOG_DEBUG, "Dialing NOT valid");
        return 0;
    }
    cpLog(LOG_DEBUG, "Dialing complete");
    cout << endl;

    //TODO delete this debugging line
    //cpLog( LOG_DEBUG, "Request url %s", digitCollector->getUrl().c_str() );

    Sptr < SipUrl > toUrl;

    try
    {
        toUrl = new SipUrl(digitCollector->getUrl());

        if (dial_phone == digitCollector->getDialMethod())
        {
            cpLog(LOG_DEBUG_STACK, "setting phone # %s", toUrl->getUserValue().getData());
            toUrl->setUserValue(toUrl->getUserValue(), dial_phone);
        }
    }
    catch(SipUrlParserException &)
    {
        cpLog(LOG_ERR, "Invalid url %s", digitCollector->getUrl().c_str());
        return 0;
    }

    assert(toUrl != 0);

    string proxyServer = RgConfiguration::instance()->getProxyServer();

    cpLog(LOG_DEBUG, "proxyServer is %s", proxyServer.c_str());

    if (toUrl->getHost().length() <= 0 && proxyServer.length())
    {
        NetworkAddress na(proxyServer);

        toUrl->setHost(na.getIpName());
        if (na.getPort() > 0)
        {
            toUrl->setPort(na.getPort());
        }
    }

    //TODO clean up

    Sptr < SipUrl > proxyUrl;
    try
    {
        proxyUrl = new SipUrl("sip:" + proxyServer);
    }
    catch(SipUrlParserException &)
    {
        cpLog(LOG_ERR, "Invalid url %s", proxyServer.c_str());
        return 0;
    }

    string sipPort = RgConfiguration::instance()->getLocalSipPort();

    InviteMsg msg(toUrl, atoi(sipPort.c_str()));

    msg.setCallId(*(RgDevice::instance()->getCallId()));

    // Set the Req-Url of INVITE
    if (proxyServer.length() > 0)
    {
        // Req-Url will have Host,Port of Proxy Server
        SipRequestLine reqLine = msg.getRequestLine();

        Sptr < BaseUrl > baseUrl = reqLine.getUrl();
        assert(baseUrl != 0);

        // Assume we have a SIP_URL
        Sptr < SipUrl > reqUrl;
        reqUrl.dynamicCast(baseUrl);
        assert(reqUrl != 0);

        reqUrl->setHost(proxyUrl->getHost());
        reqUrl->setPort(proxyUrl->getPort());
        reqLine.setUrl(reqUrl);
        msg.setRequestLine(reqLine);

    }

    // Set the From of INVITE
    SipFrom from = msg.getFrom();

    from.setDisplayName(Data(RgConfiguration::instance()->getDisplayName()));
    Sptr < SipUrl > fromUrlPtr;
    fromUrlPtr.dynamicCast(from.getUrl());
    //SipUrl fromUrl = *fromUrlPtr;
    fromUrlPtr->setUserValue(Data(RgConfiguration::instance()->getUserName()),
                             Data("phone"));
    //fromUrlPtr = &fromUrl;
    from.setUrl(fromUrlPtr);
    msg.setFrom(from);

    // Set transport in Via:
    SipVia via = msg.getVia();

    msg.removeVia();
    via.setTransport(RgConfiguration::instance()->getSipTransport());
    msg.setVia(via);

    // Set Contact: header
    Sptr < SipUrl > myUrl = new SipUrl;

    myUrl->setUserValue(Data(RgConfiguration::instance()->getUserName()), Data("phone"));
    myUrl->setHost(Data(theSystem.gethostAddress()));
    myUrl->setPort(atoi(RgConfiguration::instance()->getLocalSipPort().c_str()));

    SipContact me;

    me.setUrl(myUrl);
    msg.setNumContact(0);               // Clear
    msg.setContact(me);

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < SipSdp > sipSdp;
    sipSdp.dynamicCast(msg.getContentData(0));
    if (sipSdp != 0)
    {
        SdpSession sdpDesc = sipSdp->getSdpDescriptor();

        vector < SdpMedia * >mediaList;
        mediaList = sdpDesc.getMediaList();
        vector < SdpMedia * >::iterator mediaIterator = mediaList.begin();
        MediaAttributes *mediaAttrib;

        mediaAttrib = (*mediaIterator)->getMediaAttributes();
        (*mediaIterator)->getFormatList()->clear();     //flushing all previous format settings. Setting it afresh below.
        if (mediaAttrib == 0)
        {
            mediaAttrib = new MediaAttributes();
            assert(mediaAttrib);
            (*mediaIterator)->setMediaAttributes(mediaAttrib);
        }
        ValueAttribute *attrib = new ValueAttribute();

        attrib->setAttribute("ptime");
        attrib->setValue(Data(RgConfiguration::instance()->getNetworkRtpRate()).
                         convertString().c_str());

        // add the value attribute just created to the media attribute object
        mediaAttrib->addValueAttribute(attrib);

        // Now, ULAW is no more the default payload type. G.723, 6.3 Kbps is the default one now. 

        // add the rtpmap attribute for the PCMU codec
        SdpRtpMapAttribute *rtpMapAttrib = new SdpRtpMapAttribute();

        rtpMapAttrib->setPayloadType(0);
        rtpMapAttrib->setEncodingName("PCMU");
        rtpMapAttrib->setClockRate(8000);

        // add the rtpmap attribute for the G723 codec
        // which is now the default.
        SdpRtpMapAttribute *rtpMapAttribG7 = new SdpRtpMapAttribute();

        rtpMapAttribG7->setPayloadType(4);
        rtpMapAttribG7->setEncodingName("G723");
        rtpMapAttribG7->setClockRate(8000);

        /********************************************************************
        // add the value attribute for bitrate of G723 codec
        ValueAttribute* fmtAttrib = new ValueAttribute();
        fmtAttrib->setAttribute( "fmtp" );
        fmtAttrib->setValue( "4 bitrate=6300" );
	********************************************************************/

        (*mediaIterator)->addFormat(rtpMapAttribG7->getPayloadType());
        mediaAttrib->addmap(rtpMapAttribG7);
        // mediaAttrib->addValueAttribute( fmtAttrib );

        (*mediaIterator)->addFormat(rtpMapAttrib->getPayloadType());
        mediaAttrib->addmap(rtpMapAttrib);

        sipSdp->setSdpDescriptor(sdpDesc);

        sipSdp->setRtpPort(atoi(RgConfiguration::instance()->getLocalRtpPort().c_str()));

        call->setLocalSdp(new SipSdp(*sipSdp));
        //cpLog( LOG_DEBUG, "Local SDP:\n%s", call->getLocalSdp()->encode().getData() );
    }

    // Save INVITE
    call->setRingInvite(new InviteMsg(msg));

    // Send INVITE
    timerEvent->getSipStack()->sendAsync(msg);

    Sptr < Contact > contact = new Contact(msg);
    // Set as current contact
    call->setContact(contact);
    // Add to contact list
    call->addContact(contact);

    // Change state to gwStateIPTrying
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    cout << VTime::strftime("%y/%m/%d %H:%M:%S")
         << " Calling " << toUrl->encode().getData() << endl;

    return stateMachine->findState("gwStateIPTrying");
}
