
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpPSTNAnswered.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <StatusMsg.hxx>

#include "RgHardwareEvent.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgStateMachine.hxx"
#include "RgConfiguration.hxx"
#include "RgOperator.hxx"
#include "RgTimerEvent.hxx"
#include "RgDeviceEvent.hxx"
#include "FormatSupportCheck.hxx"

#include "gwOpPSTNAnswered.hxx"

///
gwOpPSTNAnswered::gwOpPSTNAnswered()
{
}

///
gwOpPSTNAnswered::~gwOpPSTNAnswered()
{
}

///
const char *const
gwOpPSTNAnswered::name() const
{
    return "gwOpPSTNAnswered";
}

//**************************************************************************
// gwOpPSTNAnswered::process
//
// Description  : 
// This is the operator for gwStatePSTNTrying. 
// For a IP-PSTN call, it performs the following functions:
// (1) Receive  Timeout for cadence timer.
// (2) Set up local SDP based on remote SDP
// (3) Send Status response 200 OK
// (3) Change state to gwStateInCall
//**************************************************************************
const Sptr < State >
gwOpPSTNAnswered::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpPSTNAnswered operation");

    Sptr < RgTimerEvent > timerEvent;
    timerEvent.dynamicCast(event);
    if (timerEvent == 0)
    {
        //cpLog( LOG_DEBUG, "Not an timer event  " );
        return 0;
    }

    // Catch the Timeout event
    Sptr < RgOperator > RgOpr;
    if (RgOpr->isTimeout(timerEvent) == false)
    {
        return 0;
    }

    if (strcmp("gwOpStartTimer", timerEvent->getCreatedByOpr()) == 0)
    {
        cpLog(LOG_DEBUG, "Time out not for Ringback cadence ");
//         cout << "Time out not for Ringback cadence " << endl;
        return 0;
    }
    cpLog(LOG_DEBUG, "Timeout for PSTN Ringback cadence ");
    cout << "PSTN Answered." << endl;

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < InviteMsg > msg = call->getRingInvite();
    assert(msg != 0);

    // Get remote SDP from message
    Sptr < SipSdp > remoteSdp;
    remoteSdp.dynamicCast(msg->getContentData(0));
    if (remoteSdp == NULL)
    {
        cpLog(LOG_WARNING, "No remote SDP in INVITE");
    }
    else                                // if( remoteSdp->isValidSdp() )
    {
        call->setRemoteSdp(new SipSdp(*remoteSdp));
    }
    StatusMsg status(*msg, 200 /*OK*/);

    // Add myself to the contact
    Sptr < SipUrl > myUrl = new SipUrl;
    myUrl->setUserValue(Data(RgConfiguration::instance()->getUserName()));
    myUrl->setHost(Data(theSystem.gethostAddress()));
    myUrl->setPort(atoi(RgConfiguration::instance()->getLocalSipPort().c_str()));
    SipContact me;

    me.setUrl(myUrl);
    status.setNumContact(0);            // Clear
    status.setContact(me);

    // Create local SDP based on remote SDP
    Sptr < SipSdp > localSdp;
    localSdp.dynamicCast(status.getContentData(0));

    if (localSdp != 0)
    {
        localSdp->setRtpPort(remoteSdp->getRtpPort() + 10);

        int rtpPacketSize = 20;

        SdpSession sdpDesc = localSdp->getSdpDescriptor();

        vector < SdpMedia * >mediaList;
        mediaList = sdpDesc.getMediaList();
        vector < SdpMedia * >::iterator mediaIterator = mediaList.begin();
        while (mediaIterator != mediaList.end())
        {
            if ((*mediaIterator)->getMediaType() == MediaTypeAudio)
            {
                break;
            }
            ++mediaIterator;
        }

        //***********************************************************************************
        //The following code assumes some audio media definition in the sdp.
        //Later it needs to executed only if one exists.
        //***********************************************************************************

        vector < int >*formatList = (*mediaIterator)->getFormatList();
        int checkFormat;

        if (formatList != 0)
        {
            vector < int >::iterator fmtIterator = formatList->begin();

            while (fmtIterator != formatList->end())
            {
                checkFormat = *fmtIterator;

                if (FormatSupportCheck::instance()->
                    FormatSupportedOnHardware(checkFormat))
                {
                    cpLog(LOG_DEBUG, "Hardware support available for %d Payload type",
                          checkFormat);
                    cpLog(LOG_DEBUG, "Hence, choosing this format for the session.");
                    break;
                }

                ++fmtIterator;
            }

            if (fmtIterator == formatList->end())
            {

                //*****************************************************************************
                // No format could be negotiated based on hardware capability. 
                // Hence now checking if, s/w codec supported.
                // ****************************************************************************

                fmtIterator = formatList->begin();
                while (fmtIterator != formatList->end())
                {
                    checkFormat = *fmtIterator;

                    if (FormatSupportCheck::instance()->
                        FormatSupportedOnSoftware(checkFormat))
                    {
                        cpLog(LOG_DEBUG, "Software support available for %d Payload type",
                              checkFormat);
                        cpLog(LOG_DEBUG, "Hence, choosing this format for the session.");
                        break;
                    }

                    ++fmtIterator;
                }
            }

            if (fmtIterator == formatList->end())
                checkFormat = 0;

            formatList->clear();

        }

        // using the codec negotiated above.
        (*mediaIterator)->addFormat(checkFormat);

        MediaAttributes *mediaAttrib;

        mediaAttrib = (*mediaIterator)->getMediaAttributes();
        if (mediaAttrib != 0)
        {
            vector < ValueAttribute * >*valueAttribList =
                mediaAttrib->getValueAttributes();
            vector < ValueAttribute * >::iterator attribIterator =
                valueAttribList->begin();
            while (attribIterator != valueAttribList->end())
            {
                char *attribName = (*attribIterator)->getAttribute();

                if (strcmp(attribName, "ptime") == 0)
                {
                    rtpPacketSize = Data((*attribIterator)->getValue()).convertInt();
                    break;
                }
                attribIterator++;
            }
            mediaAttrib->flushValueAttributes();
            mediaAttrib->flushrtpmap();

            ValueAttribute *attrib = new ValueAttribute();

            attrib->setAttribute("ptime");
            attrib->setValue(Data(rtpPacketSize).convertString().c_str());

            // start format specific value attributes  addition.
            // create rtpmap as needed.
            bool doneAddingAttributes = false;

            if (checkFormat == rtpPayloadPCMU)
            {

                mediaAttrib->flushValueAttributes();
                mediaAttrib->flushrtpmap();

                SdpRtpMapAttribute *rtpMapAttrib = new SdpRtpMapAttribute();

                rtpMapAttrib->setPayloadType(0);
                rtpMapAttrib->setEncodingName("PCMU");
                rtpMapAttrib->setClockRate(8000);

                // add the value attribute just created to the media attribute object
                mediaAttrib->addValueAttribute(attrib);
                mediaAttrib->addmap(rtpMapAttrib);

                doneAddingAttributes = true;
            }
            else
            {
                vector < SdpRtpMapAttribute * >reqRtpMap;
                vector < SdpRtpMapAttribute * >::iterator rtpMapIter;

                reqRtpMap = *(mediaAttrib->getmap());
                rtpMapIter = reqRtpMap.begin();
                SdpRtpMapAttribute *tempRtpMap;
                int tempPayload;

                while (rtpMapIter != reqRtpMap.end())
                {
                    tempRtpMap = *rtpMapIter;
                    tempPayload = tempRtpMap->getPayloadType();
                    if (tempPayload == checkFormat)
                    {
                        // add the rtpmap attribute for the default codec
                        SdpRtpMapAttribute *rtpMapAttrib = new SdpRtpMapAttribute();

                        rtpMapAttrib->setPayloadType(tempPayload);
                        rtpMapAttrib->setEncodingName(tempRtpMap->getEncodingName());
                        rtpMapAttrib->setClockRate(tempRtpMap->getClockRate());

                        mediaAttrib->flushValueAttributes();
                        mediaAttrib->flushrtpmap();

                        // add the value attribute just created to the media attribute object
                        mediaAttrib->addValueAttribute(attrib);
                        mediaAttrib->addmap(rtpMapAttrib);

                        doneAddingAttributes = true;
                        break;
                    }
                    rtpMapIter++;
                }
                if (!doneAddingAttributes)
                {
                    mediaAttrib->flushValueAttributes();
                    mediaAttrib->flushrtpmap();
                    // add the value attribute just created to the media attribute object
                    mediaAttrib->addValueAttribute(attrib);
                }
            }                           // else

        }

        localSdp->setSdpDescriptor(sdpDesc);

        call->setLocalSdp(new SipSdp(*localSdp));
    }
    else
    {
        // May not have SDP in original INVITE for 3rd party call control
        SipSdp sdp;
        SdpSession sdpDesc;

        Data connAddr = theSystem.gethostAddress();
        SdpConnection connection;

        connection.setUnicast(connAddr.convertString().c_str());
        sdpDesc.setConnection(connection);

        SdpMedia *media = new SdpMedia;

        media->setPort(23456);          // which port to use when making a call
        // RTP doesn't have a suggestd number

        sdpDesc.addMedia(media);
        //TODO Use NTP time
        srandom((unsigned int)time(NULL));
        int verStr = random();

        //TODO Use NTP time
        srandom((unsigned int)time(NULL));
        int sessionStr = random();

        sdpDesc.setVersion(verStr);
        sdpDesc.setSessionId(sessionStr);

        // To further perform any other operation on that particular media
        // item, use loc.

        MediaAttributes *mediaAttrib = new MediaAttributes();

        assert(mediaAttrib);
        media->setMediaAttributes(mediaAttrib);

        // create the new value attribute object
        ValueAttribute *attrib = new ValueAttribute();

        // set the attribute and its value
        attrib->setAttribute("ptime");
        attrib->setValue(Data(RgConfiguration::instance()->getNetworkRtpRate()).
                         convertString().c_str());

        // add the rtpmap attribute for the default codec
        SdpRtpMapAttribute *rtpMapAttrib = new SdpRtpMapAttribute();

        rtpMapAttrib->setPayloadType(0);
        rtpMapAttrib->setEncodingName("PCMU");
        rtpMapAttrib->setClockRate(8000);

        // add the value attribute just created to the media attribute object
        mediaAttrib->addValueAttribute(attrib);
        mediaAttrib->addmap(rtpMapAttrib);

        sdp.setSdpDescriptor(sdpDesc);
        sdp.setUserName(theSystem.getUserName());

        sdp.setSessionName(theSystem.getSessionName());
        sdp.setAddress(theSystem.gethostAddress());
        status.setContentData(&sdp, 0);

        call->setLocalSdp(new SipSdp(sdp));
    }
    timerEvent->getSipStack()->sendReply(status);

    // Change state to gwStateInCall
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    return stateMachine->findState("gwStateInCall");
}
