
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpPSTNDialNum.cxx,v 1.3 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <StatusMsg.hxx>

#include "RgHardwareEvent.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgStateMachine.hxx"
#include "RgConfiguration.hxx"

#include "gwOpPSTNDialNum.hxx"

///
gwOpPSTNDialNum::gwOpPSTNDialNum()
{
}

///
gwOpPSTNDialNum::~gwOpPSTNDialNum()
{
}

///
const char *const
gwOpPSTNDialNum::name() const
{
    return "gwOpPSTNDialNum";
}

//**************************************************************************
// gwOpPSTNDialNum::process
//
// Description  : 
// This is the operator for gwStatePSTNTrying. 
// For a IP-PSTN call, it performs the following functions:
// (1) Check for the Dial Tone
// (2) Validate the PSTN number requested
// (3) Map the PSTN number as required for local and international dialing
// (4) Dial the PSTN Number
// (5) Change state to gwStatePSTNTrying
//**************************************************************************
const Sptr < State >
gwOpPSTNDialNum::process(const Sptr < SipProxyEvent > event)
{
    unsigned int i;

    cpLog(LOG_DEBUG, "gwOpPSTNDialNum operation");

    //Check for the Dial tone event
    Sptr < RgDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast(event);

    if (deviceEvent == 0)
    {
        return 0;
    }

    if (deviceEvent->type != DeviceEventPSTNDialtone)
    {
        return 0;
    }
    cpLog(LOG_DEBUG, "Line is up.Dialtone available");

    //Check if the PSTN number is valid
    extern char pstnnum[20];

    cpLog(LOG_DEBUG, "PSTN number is %s", pstnnum);
    if (pstnnum == NULL)
    {
        cpLog(LOG_ERR, "PSTN number is NULL");
        exit(1);
    }
    for (i = 0; i < strlen(pstnnum); i++)
    {
        if (!isdigit(pstnnum[i]))
        {
            cpLog(LOG_ERR, "PSTN number is not numeric");
            exit(1);
        }
    }

    /*
     *      Following If block for PSTN calls (local & International)
     */
    if ('9' == pstnnum[0])
    {
        if ('1' == pstnnum[1])
        {
            // International calling request. Need to dial '00', followed by number.
            pstnnum[0] = '0';
            pstnnum[1] = '0';
        }
        else
        {
            for (i = 0; i < strlen(pstnnum); i++)
            {
                pstnnum[i] = pstnnum[i + 1];
            }
            pstnnum[i] = '\0';
        }
    }
    cpLog(LOG_DEBUG, "Dialing out: %s", pstnnum);

    cout << "Dialing out: " << pstnnum << endl;

    // Dial PSTN number
    unsigned int cnt;
    char cdigit[2];
    DeviceSignalType deviceSignal;

    cdigit[1]='\0';
    for (cnt = 0; cnt < strlen(pstnnum); cnt++)
    {
        cdigit[0] = pstnnum[cnt];
        switch (atoi(cdigit))
        {
            case 0:
                {
                    deviceSignal = DeviceSignalPSTNDtmf0;
                }
                break;
            case 1:
                {
                    deviceSignal = DeviceSignalPSTNDtmf1;
                }
                break;
            case 2:
                {
                    deviceSignal = DeviceSignalPSTNDtmf2;
                }
                break;
            case 3:
                {
                    deviceSignal = DeviceSignalPSTNDtmf3;
                }
                break;
            case 4:
                {
                    deviceSignal = DeviceSignalPSTNDtmf4;
                }
                break;
            case 5:
                {
                    deviceSignal = DeviceSignalPSTNDtmf5;
                }
                break;
            case 6:
                {
                    deviceSignal = DeviceSignalPSTNDtmf6;
                }
                break;
            case 7:
                {
                    deviceSignal = DeviceSignalPSTNDtmf7;
                }
                break;
            case 8:
                {
                    deviceSignal = DeviceSignalPSTNDtmf8;
                }
                break;
            case 9:
                {
                    deviceSignal = DeviceSignalPSTNDtmf9;
                }
                break;
            default:
                {
                    cpLog(LOG_DEBUG, "Unrecognized digit signal");
                }
                break;
        }                               // end switch

        Sptr < RgHardwareEvent > signal;

        signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
        signal->type = HardwareSignalType;
        signal->signalOrRequest.signal = deviceSignal;

        RgDevice::getDeviceQueue()->add(signal);

    }                                   // end for loop

    // Change state to gwStatePSTNTrying
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    cpLog(LOG_DEBUG, "Change state to gwStatePSTNTrying");
    return stateMachine->findState("gwStatePSTNTrying");

}
