
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpPSTNNoAnswer.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <StatusMsg.hxx>

#include "RgHardwareEvent.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgStateMachine.hxx"
#include "RgConfiguration.hxx"
#include "RgDeviceEvent.hxx"
#include "RgCallContainer.hxx"
#include "RgOperator.hxx"
#include "RgTimerEvent.hxx"

#include "gwOpPSTNNoAnswer.hxx"

///
gwOpPSTNNoAnswer::gwOpPSTNNoAnswer()
{
}

///
gwOpPSTNNoAnswer::~gwOpPSTNNoAnswer()
{
}

///
const char *const
gwOpPSTNNoAnswer::name() const
{
    return "gwOpPSTNNoAnswer";
}

//**************************************************************************
// gwOpPSTNNoAnswer::process
//
// Description  : 
// This is the operator for gwStatePSTNTrying. 
// For a IP-PSTN call, it performs the following functions:
// (1) Receive Timeout on PSTN.
// (1a) Stop the cadence timer.
// (2) Send 486 Busy to IP user
// (2a) Disconnect on PSTN side
// (3) Change state to gwStateIdle
//*************************************************************************/
const Sptr < State >
gwOpPSTNNoAnswer::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpPSTNNoAnswer operation");

    // Receive No answer event  
    Sptr < RgDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast(event);
    if (deviceEvent == 0)
    {
        return 0;
    }

    if (deviceEvent->type != DeviceEventPSTNNoAnswer)
    {
        return 0;
    }

    cpLog(LOG_DEBUG, "Received No ANswer from Quicknet cadence count");

    // stop the cadence timer.
    Sptr < RgOperator > RgOpr;
    RgOpr->cancelTimer(event);
    cpLog(LOG_DEBUG, "Ringback Cadence Timer is canceled here");

    cpLog(LOG_DEBUG, "Send 486 Busy");

    Sptr < RgCallInfo > call;
    call.dynamicCast(deviceEvent->getCallInfo());
    assert(call != 0);

    Sptr < InviteMsg > msg = call->getRingInvite();
    assert(msg != 0);

    StatusMsg status(*msg, 486 /*Busy */ );

    deviceEvent->getSipStack()->sendReply(status);

    // Send Release signal to PSTN user
    // i.e., send On Hook event on the PSTN line
    Sptr < RgHardwareEvent > signalOnHook =
        new RgHardwareEvent(RgDevice::getDeviceQueue());

    signalOnHook->type = HardwareSignalType;
    signalOnHook->signalOrRequest.signal = DeviceSignalPSTNOnHook;

    RgDevice::getDeviceQueue()->add(signalOnHook);

    cpLog(LOG_DEBUG, "PSTN goes ON_HOOK in NoAnswer Operation");

    // Remove call info from call container
    Sptr < RgCallContainer > calls;
    calls.dynamicCast(event->getCallContainer());
    assert(calls != 0);
    calls->deleteCall(msg->getCallId());
    // Clear call id in device
    RgDevice::instance()->setCallId(0);

    // Change state to gwStateIdle
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    cout << "READY" << endl;
    return stateMachine->findState("gwStateIdle");

}
