static const char *const version =
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

    "$Id: gwOpPSTNRinging.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <StatusMsg.hxx>

#include "RgHardwareEvent.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgStateMachine.hxx"

#include "RgConfiguration.hxx"
#include "RgTimerEvent.hxx"
#include "RgOperator.hxx"

#include "gwOpPSTNRinging.hxx"

///
gwOpPSTNRinging::gwOpPSTNRinging()
{
}

///
gwOpPSTNRinging::~gwOpPSTNRinging()
{
}

///
const char *const
gwOpPSTNRinging::name() const
{
    return "gwOpPSTNRinging";
}

//**************************************************************************
// Description  : 
// This is the operator for gwStatePSTNTrying. 
// For a IP-PSTN call, it performs the following functions:
// (1) Receive Ringback from PSTN user
// (2) Send 180 Ringing to IP user
// (3) Stop the Invalid Number timeout.
// (3) Start the ringback  cadence Timer.
//**************************************************************************
const Sptr < State >
gwOpPSTNRinging::process(const Sptr < SipProxyEvent > event)
{

    cpLog(LOG_DEBUG, "gwOpPSTNRinging operation");

    // Receive Ringback from PSTN user
    Sptr < RgDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast(event);

    if (deviceEvent == 0)
    {
        return 0;
    }

    if (deviceEvent->type != DeviceEventPSTNRingback)
    {
        return 0;
    }
    cpLog(LOG_DEBUG, "Received Ringback from PSTN");
    cout << "Received Ringback from PSTN" << endl;

    // Send 180 Ringing to IP user
    cpLog(LOG_DEBUG, "Send 180 Ringing");

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < InviteMsg > msg = call->getRingInvite();
    assert(msg != 0);

    StatusMsg status(*msg, 180 /*Ringing */ );

    deviceEvent->getSipStack()->sendReply(status);

    // Stop the InvalidNum Timer  
    Sptr < RgOperator > RgOpr;
    RgOpr->cancelTimer(event);
    cpLog(LOG_DEBUG, "InvalidNum Timer is canceled here");

    usleep(100);
    // changed , earlier used to start timer for no answer.
    // Start the timer for cadence detection. 
    //Sptr<RgOperator> RgOpr;
    if (!RgOpr->
        setTimer(event, RgConfiguration::instance()->getCadenceTimeout(),
                 "gwOpPSTNRinging"))
    {
        cpLog(LOG_DEBUG, "Ringback cadence Timer not started");
    }
    else
    {
        cpLog(LOG_DEBUG, "Ringback cadence Timer started");
    }

    // No change in State
    return 0;
}
