static const char *const version =
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

    "$Id: gwOpPSTNTerminates.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <ByeMsg.hxx>

#include "RgDeviceEvent.hxx"
#include "RgDevice.hxx"
#include "RgCallInfo.hxx"
#include "RgConfiguration.hxx"
#include "RgStateMachine.hxx"
#include "RgCallContainer.hxx"
#include "ResGwDevice.hxx"

#include "gwOpPSTNTerminates.hxx"

///
gwOpPSTNTerminates::gwOpPSTNTerminates()
{
}

///
gwOpPSTNTerminates::~gwOpPSTNTerminates()
{
}

///
const char *const
gwOpPSTNTerminates::name() const
{
    return "gwOpPSTNTerminates";
}

//**************************************************************************
// gwOpPSTNTerminates::process
//
// Description  : 
// This is the operator for gwStateInCall. 
// It performs the following functions:
// (1) Receive Busy signal from PSTN
// (2a) Disconnect on PSTN side
// (2b) Send BYE to IP user
// (3) Change state to gwStateIdle
//**************************************************************************
const Sptr < State >
gwOpPSTNTerminates::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpPSTNTerminates operation");

    // Receive Busy signal from PSTN
    Sptr < RgDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast(event);
    if (deviceEvent == 0)
    {
        return 0;
    }
    if (deviceEvent->type != DeviceEventPSTNBusy)
    {
        return 0;
    }
    cpLog(LOG_DEBUG, "Received Busy from PSTN");

    // Send 486 Busy to IP user
    // cpLog( LOG_DEBUG, "Send 486 Busy" );

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    // Send Release signal to PSTN user
    // i.e., send On Hook event on the PSTN line
    Sptr < RgHardwareEvent > signalOnHook =
        new RgHardwareEvent(RgDevice::getDeviceQueue());

    signalOnHook->type = HardwareSignalType;
    signalOnHook->signalOrRequest.signal = DeviceSignalPSTNOnHook;

    RgDevice::getDeviceQueue()->add(signalOnHook);

    // Send BYE to IP user
    Sptr < Contact > contact = call->getContact();
    if (contact != 0)
    {
        SipVia sipVia;

        sipVia.setprotoVersion("2.0");
        sipVia.setHost(Data(theSystem.gethostAddress()));
        sipVia.setPort(atoi(RgConfiguration::instance()->getLocalSipPort().c_str()));
        sipVia.setTransport(RgConfiguration::instance()->getSipTransport());

        unsigned int cseq = contact->getCSeqNum();

        contact->setCSeqNum(++cseq);

        Sptr < ByeMsg > bye;

        if (contact->getStatus() != 200)
        {
            const InviteMsg & msg = contact->getInviteMsg();

            if (&msg != 0)
            {
                cpLog(LOG_DEBUG, "BYE to Caller");
                bye = new ByeMsg(msg);
            }
            else
            {
                cpLog(LOG_ERR, "No INVITE message for BYE");
            }
        }
        else                            //NOt required in the PSTN-IP scenario
        {
            const StatusMsg & msg = contact->getStatusMsg();

            if (&msg != 0)
            {
                cpLog(LOG_DEBUG,
                      "BYE to IP (called party)... RGW is caller for IP in PSTN-IP call");
                bye = new ByeMsg(msg);
            }
            else
            {
                cpLog(LOG_ERR, "No status message for BYE");
            }
        }

        bye->flushViaList();
        bye->setVia(sipVia, 0);

        SipCSeq sipCSeq = bye->getCSeq();

        sipCSeq.setCSeq(cseq);
        bye->setCSeq(sipCSeq);

        vector < SipRoute * >routeList;
        // add the route info   
        if (*(call->getRingInvite()) == contact->getInviteMsg())
        {
            cpLog(LOG_DEBUG, "adding route info for the first call");
            routeList = call->getRoute1List();
        }

        if (routeList.size() > 0)
        {
            bye->setRouteList(routeList);
            SipRoute siproute = bye->getRoute(0);

            bye->removeRoute(0);

            SipRequestLine reqLine = bye->getRequestLine();

            reqLine.setUrl(siproute.getUrl());
            bye->setRequestLine(reqLine);
        }

        deviceEvent->getSipStack()->sendAsync(*bye);
        // cout << "Sent BYE" << endl;
    }
    else
    {
        cpLog(LOG_ERR, "No current contact");
    }

    // Remove call info from call container
    Sptr < RgCallContainer > calls;
    calls.dynamicCast(event->getCallContainer());
    assert(calls != 0);

    calls->deleteCall(*(RgDevice::instance()->getCallId()));
    // Clear call id in device
    RgDevice::instance()->setCallId(0);

    // Change state to gwStateIdle
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    cout << "READY" << endl;
    return stateMachine->findState("gwStateIdle");

}
