static const char *const version =
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

    "$Id: gwOpRecvAck.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>

#include "RgDeviceEvent.hxx"
#include "RgDevice.hxx"
#include "RgCallInfo.hxx"
#include "RgConfiguration.hxx"

#include "gwOpRecvAck.hxx"

///
gwOpRecvAck::gwOpRecvAck()
{
}

///
gwOpRecvAck::~gwOpRecvAck()
{
}

///
const char *const
gwOpRecvAck::name() const
{
    return "gwOpRecvAck";
}

//***************************************************************************
//      gwOpRecvAck::process
//
//      Description     : 
//      This is the operator for gwStateInCall. 
//      For a IP-PSTN call, it performs the following functions:
//      (1) Receive ACK from IP user
//
//***************************************************************************
///
const Sptr < State >
gwOpRecvAck::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpRecvAck operation");

    Sptr < SipEvent > sipEvent;
    sipEvent.dynamicCast(event);
    if (sipEvent == 0)
        return 0;

    Sptr < SipMsg > sipMsg = sipEvent->getSipMsg();
    assert(sipMsg != 0);

    if (sipMsg->getType() != SIP_ACK)
    {
        return 0;
    }
    // // cout << "Received ACK" << endl;

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    // Get remote SDP from call info
    Sptr < SipSdp > remoteSdp = call->getRemoteSdp();
    if (remoteSdp != 0)                 //&& remoteSdp->isValidSdp() )
    {
        // TODO For now, do nothing regardless if ACK contains sdp.
        // TODO In the future, if ACK has sdp, we should stop the previous
        // TODO media stream ans start a new one with the new sdp.
    }
    else
    {
        // Get remote SDP from ACK 
        Sptr < SipSdp > sdp;
        sdp.dynamicCast(sipMsg->getContentData(0));
        if (sdp == NULL)
        {
            cpLog(LOG_ERR, "No remote SDP in ACK");
        }
        else                            //if( sdp->isValidSdp() )
        {
            remoteSdp = new SipSdp(*sdp);
            call->setRemoteSdp(remoteSdp);
            Sptr < SipSdp > localSdp = call->getLocalSdp();
            if (localSdp != 0)
            {
                // startAudio( localSdp, remoteSdp );
            }
            else
            {
                cpLog(LOG_ERR, "No local SDP");
            }
        }
    }

    return 0;
}

///
void
gwOpRecvAck::startAudio(Sptr < SipSdp > localSdp, Sptr < SipSdp > remoteSdp)
{
    // Create a hardware event of type AudioRequest &
    // post it to the Device Queue
    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareAudioType;

    struct HardwareAudioRequest *request = &(signal->signalOrRequest.request);

    request->type = AudioStart;

    // Set remote host and port
    strcpy(request->remoteHost, remoteSdp->getConnAddress().getData());
    request->remotePort = remoteSdp->getRtpPort();

    // Set local host and port
    request->localPort = localSdp->getRtpPort();
    strcpy(request->localHost, localSdp->getConnAddress().getData());

    //TODO Why echo cancellation always true?
    request->echoCancellation = true;

    int rtpPacketSize = 20;

    SdpSession sdpDesc = localSdp->getSdpDescriptor();

    vector < SdpMedia * >mediaList;
    mediaList = sdpDesc.getMediaList();
    vector < SdpMedia * >::iterator mediaIterator = mediaList.begin();

    MediaAttributes *mediaAttrib;

    mediaAttrib = (*mediaIterator)->getMediaAttributes();
    if (mediaAttrib != 0)
    {
        vector < ValueAttribute * >*valueAttribList = mediaAttrib->getValueAttributes();
        vector < ValueAttribute * >::iterator attribIterator = valueAttribList->begin();
        while (attribIterator != valueAttribList->end())
        {
            char *attribName = (*attribIterator)->getAttribute();

            if (strcmp(attribName, "ptime") == 0)
            {
                rtpPacketSize = Data((*attribIterator)->getValue()).convertInt();
                break;
            }
            attribIterator++;
        }
    }
    request->rtpPacketSize = rtpPacketSize;
    request->sendRingback = false;

    //TODO RTP codec and packet size negotiation?
    cpLog(LOG_DEBUG, "Start audio( duplex ):");
    cpLog(LOG_DEBUG, "localHost     = %s", request->localHost);
    cpLog(LOG_DEBUG, "localPort     = %d", request->localPort);
    cpLog(LOG_DEBUG, "remoteHost    = %s", request->remoteHost);
    cpLog(LOG_DEBUG, "remotePort    = %d", request->remotePort);
    cpLog(LOG_DEBUG, "rtpPacketSize = %d", request->rtpPacketSize);

    RgDevice::getDeviceQueue()->add(signal);
}
