
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpResponseTimeout.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <InviteMsg.hxx>

#include "RgDigitTimerEvent.hxx"
#include "DigitCollector.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"
#include "RgCallInfo.hxx"
#include "RgStateMachine.hxx"
#include "RgOperator.hxx"
#include "RgTimerEvent.hxx"
#include "RgCallContainer.hxx"

#include "gwOpResponseTimeout.hxx"

///
gwOpResponseTimeout::gwOpResponseTimeout()
{
}

///
gwOpResponseTimeout::~gwOpResponseTimeout()
{
}

///
const char *const
gwOpResponseTimeout::name() const
{
    return "gwOpResponseTimeout";
}

//**************************************************************************
//      gwOpResponseTimeout::process
//
//      Description     : 
//      This is the operator for gwStateIPTrying. 
//      For a PSTN->IP call, it performs the following functions:
//      (1) Receives the response timeout event
//      (2) Plays Busy tone for 5 sec.
//      (3) Sends PSTN ON-HOOK 
//      (4) Start Release Timer.
//
//**************************************************************************
///
const Sptr < State >
gwOpResponseTimeout::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpResponseTimeout operation");

    Sptr < RgTimerEvent > timerEvent;
    timerEvent.dynamicCast(event);
    if (timerEvent == 0)
    {
        return 0;
    }

    // Catch the Timeout event
    Sptr < RgOperator > RgOpr;
    if (RgOpr->isTimeout(timerEvent) == false)
    {
        return 0;
    }
    if (strcmp("gwOpStartTimer", timerEvent->getCreatedByOpr()) != 0)
    {
        cpLog(LOG_DEBUG, "Not response timeout ");
        return 0;
    }

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < Contact > contact = call->getContact();
    assert(contact != 0);

    int status = contact->getStatus();

    if (status == 180 || status == 183)
    {
        cpLog(LOG_DEBUG, "180/183 message has already been received");
        cpLog(LOG_DEBUG, "Igore timeout event");
        return 0;
    }
    else if (status > 180)
    {
        cpLog(LOG_DEBUG, "Status > 180 message has already been received");
        cpLog(LOG_DEBUG, "Igore timeout event");
        return 0;
    }

    cout << "fail to connect to remote endpoint" << endl;

    // start playing busy tone.... 
    cpLog(LOG_DEBUG, "Start Busy Tone");
    cout << "Start Busy Tone " << endl;

    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalBusyStart;

    RgDevice::getDeviceQueue()->add(signal);

    vusleep(5000000);                   // sleep for 5 sec.

    // Send Release signal to PSTN user
    // i.e., send On Hook event on the PSTN line
    Sptr < RgHardwareEvent > signalOnHook =
        new RgHardwareEvent(RgDevice::getDeviceQueue());

    signalOnHook->type = HardwareSignalType;
    signalOnHook->signalOrRequest.signal = DeviceSignalPSTNOnHook;

    RgDevice::getDeviceQueue()->add(signalOnHook);

    //Start timer for release
    if (!RgOpr->
        setTimer(event, RgConfiguration::instance()->getRelTimeout(),
                 "gwOpResponseTimeout"))
    {
        cpLog(LOG_DEBUG, "Release Timer not started");
    }
    else
    {
        cpLog(LOG_DEBUG, "Release Timer started");
    }

    cout << "WAIT" << endl;

    // Change state to gwStateWait
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    return stateMachine->findState("gwStateWait");

}
