
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpRingCadenceTimer.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <SipEvent.hxx>
#include <InviteMsg.hxx>
#include <StatusMsg.hxx>
#include <CancelMsg.hxx>

#include "RgHardwareEvent.hxx"
#include "RgCallInfo.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"
#include "RgOperator.hxx"
#include "RgTimerEvent.hxx"
#include "RgDeviceEvent.hxx"
#include "RgCallContainer.hxx"
#include "RgTimerEvent.hxx"

#include "gwOpRingCadenceTimer.hxx"

///
gwOpRingCadenceTimer::gwOpRingCadenceTimer()
{
}

///
gwOpRingCadenceTimer::~gwOpRingCadenceTimer()
{
}

///
const char *const
gwOpRingCadenceTimer::name() const
{
    return "gwOpRingCadenceTimer";
}

//**************************************************************************
// gwOpRingCadenceTimer::process
//
// Description  : 
// This is the operator for gwStatePSTNTrying. 
// For a IP-PSTN call, it performs the following functions:
// (1) Receive  ringback cadence event from Quicknet .
// (2) Stops the ringback cadence timer.
// (3) Starts the ringback cadence timer for next cadence detection.
//**************************************************************************
const Sptr < State >
gwOpRingCadenceTimer::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpRingCadenceTimer operation");

    // Receive Cadence Timer Device event from Quicknet
    Sptr < RgDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast(event);
    if (deviceEvent == 0)
    {
        return 0;
    }
    if (deviceEvent->type != DeviceEventPSTNRingbackCadenceTimer)
    {
        return 0;
    }
    cpLog(LOG_DEBUG, "Received Ringback cadence Timer event from quicknet");

    // Stop the Cadence Timer started 
    Sptr < RgOperator > RgOpr;
    RgOpr->cancelTimer(event);
    cpLog(LOG_DEBUG, "Ringback Cadenec Timer is canceled here");

    usleep(100);

    //Start Timer
    if (!RgOpr->
        setTimer(event, RgConfiguration::instance()->getCadenceTimeout(),
                 "gwOpRingCadenceTimer"))
    {
        cpLog(LOG_DEBUG, "Ringback Cadence Timer not started");
    }
    else
    {
        cpLog(LOG_DEBUG, "Ringback cadence Timer started");
    }

    // no change in state
    return 0;

}
