
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpStopDialTone.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <cassert>

#include <cpLog.h>

#include "RgHardwareEvent.hxx"
#include "RgDevice.hxx"
#include "RgDigitTimerEvent.hxx"

#include "gwOpStopDialTone.hxx"

///
gwOpStopDialTone::gwOpStopDialTone()
{
}

//**************************************************************************
// gwOpStopDialTone::process
//
// Description  : 
// It performs the following functions:
// (1) Passes signal to the device to stop the Dial Tone.
//**************************************************************************
const Sptr < State >
gwOpStopDialTone::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpStopDialTone operation");

    Sptr < RgDeviceEvent > deviceEvent;
    deviceEvent.dynamicCast(event);
    cpLog(LOG_DEBUG, "deviceEvent->type=%d", deviceEvent->type);
    if (deviceEvent == 0)
    {
        Sptr < RgDigitTimerEvent > timerEvent;
        timerEvent.dynamicCast(event);
        if (timerEvent == 0)
        {
            return 0;
        }
    }
    else
    {
        switch (deviceEvent->type)
        {
            case DeviceEventPSTNDtmfStar:
            case DeviceEventPSTNDtmfHash:
            case DeviceEventPSTNDtmf0:
            case DeviceEventPSTNDtmf1:
            case DeviceEventPSTNDtmf2:
            case DeviceEventPSTNDtmf3:
            case DeviceEventPSTNDtmf4:
            case DeviceEventPSTNDtmf5:
            case DeviceEventPSTNDtmf6:
            case DeviceEventPSTNDtmf7:
            case DeviceEventPSTNDtmf8:
            case DeviceEventPSTNDtmf9:
                {
                }
                break;
            default:
                {
                    // for any other cases, don't stop dial tone.
                    return 0;
                }
        }                               //end of switch
    }
    cpLog(LOG_DEBUG, "Stop Dial Tone");

    // stop dial tone on hardware
    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalDialToneStop;

    RgDevice::getDeviceQueue()->add(signal);

    RgDevice::instance()->getDigitCollector()->clear();

    return 0;
}

///
gwOpStopDialTone::~gwOpStopDialTone()
{
}

///
const char *const
gwOpStopDialTone::name() const
{
    return "gwOpStopDialTone";
}
