
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const version =
    "$Id: gwOpTimeout.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include <InviteMsg.hxx>

#include "RgDigitTimerEvent.hxx"
#include "DigitCollector.hxx"
#include "RgDevice.hxx"
#include "RgConfiguration.hxx"
#include "RgCallInfo.hxx"
#include "RgStateMachine.hxx"
#include "RgOperator.hxx"
#include "RgTimerEvent.hxx"
#include "RgCallContainer.hxx"

#include "gwOpTimeout.hxx"

///
gwOpTimeout::gwOpTimeout()
{
}

///
gwOpTimeout::~gwOpTimeout()
{
}

///
const char *const
gwOpTimeout::name() const
{
    return "gwOpTimeout";
}

//**************************************************************************
// gwOpTimeout::process
//
// Description  : 
// This is the operator for gwStateWait. 
// For a PSTN->IP call, it performs the following functions:
// (1) Receives the timeout event
// (2) stop busy tone if playing
// (3) change to gwStateIdle
//**************************************************************************
const Sptr < State >
gwOpTimeout::process(const Sptr < SipProxyEvent > event)
{
    cpLog(LOG_DEBUG, "gwOpTimeout operation");

    Sptr < RgCallInfo > call;
    call.dynamicCast(event->getCallInfo());
    assert(call != 0);

    Sptr < RgTimerEvent > timerEvent;
    timerEvent.dynamicCast(event);
    if (timerEvent == 0)
    {
        return 0;
    }

    // Catch the Timeout event
    Sptr < RgOperator > RgOpr;
    if (RgOpr->isTimeout(timerEvent) == false)
    {
        // cpLog( LOG_DEBUG, "RgOpr->isTimeout(timerEvent) == false " );
        return 0;
    }

    if ((strcmp("gwOpIP3xx", timerEvent->getCreatedByOpr()) != 0) &&
        (strcmp("gwOpIP4xx", timerEvent->getCreatedByOpr()) != 0) &&
        (strcmp("gwOpIP5xx", timerEvent->getCreatedByOpr()) != 0) &&
        (strcmp("gwOpIP6xx", timerEvent->getCreatedByOpr()) != 0) &&
        (strcmp("gwOpIPNoAnswer", timerEvent->getCreatedByOpr()) != 0) &&
        (strcmp("gwOpResponseTimeout", timerEvent->getCreatedByOpr()) != 0) &&
        (strcmp("gwOpIPEndsCall", timerEvent->getCreatedByOpr()) != 0) &&
        (strcmp("gwOpDialError", timerEvent->getCreatedByOpr()) != 0))
    {
        cpLog(LOG_DEBUG,
              "Time out not set by gwOpIP[3-6]xx, NoAnswer, ResponseTimeout, IPEndsCall, DialError");
//         // cout << "Time out not set by gwOpIP[3-6]xx, NoAnswer, ResponseTimeout, IPEndsCall, DialError" << endl;
        return 0;
    }
    cpLog(LOG_DEBUG, "Timeout for Release(pstn-ip) ");

    // stop the busy tone
    cpLog(LOG_DEBUG, "Stop Busy Tone");
    Sptr < RgHardwareEvent > signal = new RgHardwareEvent(RgDevice::getDeviceQueue());
    signal->type = HardwareSignalType;
    signal->signalOrRequest.signal = DeviceSignalBusyStop;

    RgDevice::getDeviceQueue()->add(signal);

    usleep(500);                        // 0.5 ms

    // Remove call info from call container
    Sptr < RgCallContainer > calls;
    calls.dynamicCast(event->getCallContainer());
    assert(calls != 0);

    cpLog(LOG_DEBUG, "Assert calls not failed");
    calls->deleteCall(*(RgDevice::instance()->getCallId()));

    // Clear call id in device
    RgDevice::instance()->setCallId(0);
    cpLog(LOG_DEBUG, "Call details deleted");

    // Change state to gwStateIdle
    Sptr < RgStateMachine > stateMachine;
    stateMachine.dynamicCast(event->getCallInfo()->getFeature());
    assert(stateMachine != 0);

    cout << "READY" << endl;
    return stateMachine->findState("gwStateIdle");

}
