
/* ====================================================================
 * The Tata Infotech Software License, Version 1.0 
 * 
 * Copyright (c) 2002 Tata Infotech Ltd.  All rights reserved.
 * 
 * Redistribution and/or use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "Tata Infotech Ltd.", and "SIP based Residential Gateway"
 *    must not be used to endorse or promote products derived from this
 *    software without prior written permission. For written permission,
 *    please contact atg.voip@tatainfotech.com
 * 
 * 4. Products derived from this software may not be called "SIP based
 *    Residential Gateway", nor may "SIP based Residential Gateway" appear
 *    in their name, without prior written permission of Tata Infotech Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY WHATSOEVER AND ANY
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT
 * ARE DISCLAIMED.  
 * 
 * IN NO EVENT SHALL TATA INFOTECH LTD.  OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Tata Infotech Ltd.
 * For more information on Tata Infotech Ltd.,  please see
 * <http://www.tatainfotech.com>.
 *
 */

static const char *const rg_cxx_Version =
    "$Id: rg.cxx,v 1.2 2002/04/05 08:07:42 manish Exp $";

#include "global.h"
#include "RgCommandLine.hxx"
#include "Daemon.hxx"
#include "cpLog.h"
#include "RgConfiguration.hxx"
#include "RgBuilder.hxx"
#include "ResGateway.hxx"

#include "FormatSupportCheck.hxx"

///
int
main(int argc, char *argv[])
{
    cpLogSetLabel(argv[0]);

    RgCommandLine::instance((const int)argc, (const char **)argv);

    if (RgCommandLine::instance()->getBoolOpt("daemon"))
    {
        // TODO set cpLog to use syslog

        assert(Daemon() >= 0);
    }

    cpLogSetPriority(RgCommandLine::instance()->getIntOpt("loglevel"));

    // Turn off retransmission
    if (RgCommandLine::instance()->getIntOpt("retransmit"))
    {
        SipTransceiver::reTransOn();
    }
    else
    {
        SipTransceiver::reTransOff();
    }

    if (RgCommandLine::instance()->getIntOpt("retransinitial") != 500 ||
        RgCommandLine::instance()->getIntOpt("retransmax") != 4000)
    {
        SipTransceiver::setRetransTime(RgCommandLine::instance()->
                                       getIntOpt("retransinitial"),
                                       RgCommandLine::instance()->
                                       getIntOpt("retransmax"));
    }

    cpLog(LOG_NOTICE, "Compiled at " __DATE__ " " __TIME__);

    const string cfgStr = RgCommandLine::instance()->getStringOpt("cfgfile");
    FILE *cfgFile = fopen(cfgStr.c_str(), "r");

    if (cfgFile == 0)
    {
        cerr << "can not open " << cfgStr << endl;
        cerr << "Usage: " << argv[0] << " " << appUsage << endl;
        exit(0);

    }
    else
    {
        fclose(cfgFile);
        RgConfiguration::instance(cfgStr);
    }

    Sptr < RgBuilder > rgBuilder = new RgBuilder;

    ResGateway rg(rgBuilder,
                  Data(RgConfiguration::instance()->getLocalSipPort()).convertInt());

    // Added for assisting in sdp negotiation
    FormatSupportCheck::instance(1);

    rg.run();

    rg.join();

    return 0;
}                                       // rg main()
